#-*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--
"""
SQLite FTS based search engine plugin.
Zero dependency.
For usage, look at the unit tests.
"""

import sqlite3
import os.path
import re

unialpha = re.compile('[\W_]+', re.UNICODE)


class SQLiteFTSQueryMapper(object):

    def match(self, field, value):
        query = '%s match ?' % field.name
        # sqlite does not analyse, do it ourselves
        return (query, (unialpha.sub(u' ', value) + u'*',))

    def matchany(self, schema, value):
        query = '%s match ?' % schema.type_name
        return (query, (unialpha.sub(u' ', value) + u'*',))

    def eq(self, field, value):
        query = '%s = ?' % field.name
        return (query, (value,))

    def gt(self, field, value):
        query = '%s > ?' % field.name
        return (query, (value,))

    def gte(self, field, value):
        query = '%s >= ?' % field.name
        return (query, (value,))

    def lt(self, field, value):
        query = '%s < ?' % field.name
        return (query, (value,))

    def lte(self, field, value):
        query = '%s <= ?' % field.name
        return (query, (value,))

    def in_(self, field, value):
        query = '%s in (%s)' % (field.name,
                                ','.join(('?',) * len(value)))
        return (query, value)

# FTS does not behave like normal SQL for match operations.
# TODO: rewrite accordingly

    def and_(self, exp1, exp2):
        query = '(%s and %s)' % (exp1[0], exp2[0])
        params = exp1[1] + exp2[1]
        return (query, params)

    def or_(self, exp1, exp2):
        query = '(%s or %s)' % (exp1[0], exp2[0])
        params = exp1[1] + exp2[1]
        return (query, params)


class SQLiteFTSSchemaMapper(object):

    def __init__(self, db_cursor):
        self.db_cursor = db_cursor
        self.mappings = {}

    # Schema API

    def define(self, schema_name):
        self.mappings[schema_name] = {
            'notindexed': ['notindexed=id'],
            'fields': []
        }

    def define_field(self, schema_name, field_type, name, indexed, stored):
        # SQLite fts tables are not typed, ignoring field_type
        # SQLite fts always stores, ignoring stored.
        mapping = self.mappings[schema_name]
        if not indexed:
            mapping['notindexed'].append("notindexed=" + name)
        mapping['fields'].append(name)

    # specific API

    def create(self):
        for schema_name, mapping in self.mappings.iteritems():
            self.db_cursor.execute('drop table if exists %s' % schema_name)
            slversion = sqlite3.sqlite_version_info[:3]
            if slversion > (3, 8, 0):
                self.db_cursor.execute(
                    '''CREATE VIRTUAL TABLE %s USING fts4(id, %s, prefix="3,5,7", %s);''' %
                    (schema_name,
                     ','.join(mapping['fields']),
                     ','.join(mapping['notindexed']))
                )
            elif slversion > (3, 7, 7):
                print "Warning: older version of sqlite3 detected, please upgrade to sqlite 3.8.0 or newer."
                self.db_cursor.execute(
                    '''CREATE VIRTUAL TABLE %s USING fts4(id, %s, prefix="3,5,7");''' %
                    (schema_name,
                     ','.join(mapping['fields']))
                )
            else:
                raise ImportError('Your version of sqlite3 is too old, please upgrade to sqlite 3.8.0 or newer.')


class IndexCursor(object):

    def __init__(self,  db_cursor):
        self.db_cursor = db_cursor
        self.query = ''
        self.params = []

    # Document API

    def insert(self, schema_name, docid, **fields):
        # neither schema nor fields keys are user inputs, so this is safe
        self.query = (
            'insert into %s(id,%s) values (%s)' %
            (
                schema_name,
                ','.join(fields.keys()),
                ','.join(('?',) * (len(fields) + 1))
            )
        )
        self.params = [docid] + [value for value in fields.itervalues()]

    def update(self, schema_name, docid, **fields):
        qset = []
        for name, value in fields.iteritems():
            qset.append("%s = ?" % name)
            self.params.append(value)
        self.query = 'update %s set %s where id=?' % (
            schema_name, ', '.join(qset))
        self.params.append(docid)

    # Query API

    def search(self, schema_name, fields_to_load, mapped_query, limit):
        """
        ``fields_to_load`` is  a list of field names.
        """
        where, params = mapped_query
        self.query = 'select id as docid, %s from %s where %s limit %s' % (
            ', '.join(field for field in fields_to_load),
            schema_name, where, limit
        )
        self.params = params

    def get_results(self, result_factory):
        self.execute()
        fields = [d[0].lower() for d in self.db_cursor.description]
        # no score function in sqlite, so every result is given weight 1
        scored_results = [(1, result_factory(**dict(zip(fields, row))))
             for row in self.db_cursor.fetchall()]
        return scored_results

    # Specific API

    def execute(self):
        self.db_cursor.execute(self.query, self.params)


class SQLiteFTSEngine(object):

    '''
    Basic search engine, weakly typed, every field is indexed.
    Sufficient in most cases.
    '''

    # make it compatible with services
    LOAD_PRIORITY = 30

    def __init__(self, index, index_folder):
        assert(index.isalnum())
        self.init_state(index, index_folder)

    def init_state(self, collection, index_folder):
        self.collection = collection
        self.index_folder = index_folder
        self.connection = sqlite3.connect(
            os.path.join(index_folder, collection + '.fts'))
        self._cursor = None
        self.mapper = SQLiteFTSQueryMapper()

    # be persistence friendly
    def __getstate__(self):
        return (self.collection, self.index_folder)

    def __setstate__(self, state):
        self.init_state(*state)

    def _get_cursor(self):
        if not self._cursor:
            self._cursor = self.connection.cursor()
        return self._cursor

    def _dropdb(self):
        if self.connection:
            self.connection.close()
            self.connection = None
            self._cursor = None
        db = os.path.join(self.index_folder, self.collection + '.fts')
        os.unlink(db)

    def add_document(self, document):
        '''
        Add a document to the data store, in collection (a.k.a. index)
        `collection`, under the document type (a.k.a. schema) `schema`.

        '''
        index_cursor = IndexCursor(self._get_cursor())
        document.save(index_cursor)
        index_cursor.execute()

    def delete_document(self, schema, docid):
        '''
        Remove document from index and storage.
        '''
        c = self._get_cursor()
        c.execute('delete from %s where id=?' % schema.type_name, (docid,))

    def update_document(self, document):
        '''Update document'''
        index_cursor = IndexCursor(self._get_cursor())
        document.save(index_cursor, update=True)
        index_cursor.execute()

    def commit(self, sync=False):
        '''``sync`` option is ignored by this engine'''
        if not self._cursor:
            # no operation in progress
            return
        self.connection.commit()
        self._cursor.close()
        self._cursor = None

    def cancel(self):
        '''
        Forget documents added since last commit'''
        if self._cursor:
            self._cursor.close()
            self._cursor = None

    def search(self, query, size=20):
        '''
        Search the database.
        '''
        index_cursor = IndexCursor(self._get_cursor())
        return query.search(index_cursor, self.mapper, size)

    def delete_collection(self):
        self._dropdb()

    def create_collection(self, schemas):
        '''
        Init the collections the first time.
        Just use once! Or you'll have to reindex all your documents.
        `schemas` is a list of Document classes or Schema instances.
        '''
        c = self._get_cursor()
        mapper = SQLiteFTSSchemaMapper(c)
        for schema in schemas:
            schema.map(mapper)
        mapper.create()
        self.commit()



# -*- coding:utf-8 -*-
# --
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
# --

from nagare.i18n import _, _N, _L
from nagare import ajax, component, presentation, security, var

from kansha import notifications
from kansha.toolbox import overlay, remote
from kansha.board.boardconfig import WeightsSequenceEditor

from .boardsmanager import BoardsManager
from .comp import (Board, BoardDescription, BoardMember,
                   Icon)
from .comp import (BOARD_PRIVATE, BOARD_PUBLIC, BOARD_SHARED,
                   COMMENTS_OFF, COMMENTS_PUBLIC, COMMENTS_MEMBERS,
                   VOTES_OFF, VOTES_PUBLIC, VOTES_MEMBERS,
                   WEIGHTING_FREE, WEIGHTING_LIST, WEIGHTING_OFF)
from .boardconfig import BoardBackground, BoardConfig, BoardLabels, BoardProfile, BoardWeights

VISIBILITY_ICONS = {
    BOARD_PRIVATE: 'icon-lock',
    BOARD_PUBLIC: 'icon-unlocked',
    BOARD_SHARED: 'icon-earth'
}

BACKGROUND_POSITIONS = [
    ('fill', _L(u"fill")),
    ('fit', _L(u"fit")),
    ('stretch', _L(u"stretch")),
    ('tile', _L(u"tile")),
    ('center', _L(u"center"))
]


@presentation.render_for(Board, model="menu")
def render_Board_menu(self, h, comp, *args):
    with h.div(class_='nav-menu', onclick='YAHOO.kansha.app.toggleMainMenu(this)'):
        with h.ul(class_='actions large'):
            h << h.li(h.a(self.icons['preferences']).action(self.show_preferences))
            if security.has_permissions('edit', self):
                h << h.li(h.a(self.icons['add_list']).action(self.add_list))
                h << h.li(h.a(self.icons['edit_desc']).action(self.edit_description))
            if security.has_permissions('manage', self):
                h << h.li(h.a(self.icons['save_template']).action(self.save_template, comp))

            h << h.li(h.SyncRenderer().a(self.icons['export']).action(self.export))
            h << h.li(h.a(self.icons['history']).action(self.show_actionlog))

            if security.has_permissions('manage', self):
                h << h.li(h.SyncRenderer().a(
                    self.icons['archive'],
                    onclick=(
                        'return confirm(%s)' %
                        ajax.py2js(
                            _("This board will be archived. Are you sure?")
                        ).decode('UTF-8')
                    )
                ).action(self.archive, comp))
            else:
                h << h.li(h.SyncRenderer().a(
                    self.icons['leave'],
                    onclick=(
                        "return confirm(%s)" %
                        ajax.py2js(
                            _("You won't be able to access this board anymore. Are you sure you want to leave it anyway?")
                        ).decode('UTF-8')
                    )
                ).action(self.leave, comp))

        h << h.span(_(u'Board'), class_="menu-title", id='board-nav-menu')
    h << self.modal
    return h.root


@presentation.render_for(Board)
def render_Board(self, h, comp, *args):
    """Main board renderer"""
    security.check_permissions('view', self)
    self.refresh_on_version_mismatch()
    self.card_filter.reload_search()
    h.head.css_url('css/themes/board.css?v=2c')
    h.head.css_url('css/themes/%s/board.css?v=2c' % self.theme)

    title = '%s - %s' % (self.get_title(), self.app_title)
    h.head << h.head.title(title)
    if security.has_permissions('edit', self):
        h << comp.render(h.AsyncRenderer(), "menu")

    with h.div(class_='board'):
        if self.background_image_url:
            h << {'class': 'board ' + self.background_image_position,
                  'style': 'background-image:url(%s)' % self.background_image_url}
        with h.div(class_='header'):
            with h.div(class_='board-title', style='color: %s' % self.title_color):
                h << self.title.render(h.AsyncRenderer(), 0 if security.has_permissions('edit', self) else 'readonly')
            h << comp.render(h, 'switch')
        with h.div(class_='bbody'):
            h << comp.render(h.AsyncRenderer(), self.model)
    return h.root


@presentation.render_for(Board, 'switch')
def render_Board_item(self, h, comp, *args):

    with h.div(id='switch_zone'):
        if self.model == 'columns':
            # card search
            h << self.search_input
            # Switch view
            h << h.SyncRenderer().a(
                h.i(class_='icon-calendar'),
                title=_('Calendar mode'),
                class_='btn icon-btn ',
                style='color: %s' % self.title_color).action(self.switch_view)
            h << h.SyncRenderer().a(
                h.i(class_='icon-list'),
                title=_('Board mode'),
                class_='btn icon-btn disabled selected',
                style='color: %s' % self.title_color)
        else:
            h << h.SyncRenderer().a(
                h.i(class_='icon-calendar'),
                title=_('Calendar mode'),
                class_='btn icon-btn disabled selected',
                style='color: %s' % self.title_color)
            h << h.SyncRenderer().a(
                h.i(class_='icon-list'),
                title=_('Board mode'),
                class_='btn icon-btn',
                style='color: %s' % self.title_color).action(self.switch_view)

    return h.root


@presentation.render_for(Board, 'item')
def render_Board_item(self, h, comp, *args):
    def answer():
        comp.answer(self.data.id)

    url = self.data.url
    with h.li:
        h << h.SyncRenderer().a(
            h.i(' ', class_=VISIBILITY_ICONS[self.data.visibility]),
            self.data.title,
            href=url, class_="boardItemLabel", title=self.data.description)

        with h.div(class_='actions'):
            h << self.comp_members.render(h, 'members')

            if security.has_permissions('manage', self):
                h << h.a(
                    h.i(class_='ico-btn icon-box-add'),
                    class_='archive',
                    title=_(u'Archive "%s"') % self.data.title
                ).action(self.archive, comp)
            elif security.has_permissions('leave', self):
                onclick = 'return confirm("%s")' % _("You won't be able to access this board anymore. Are you sure you want to leave it anyway?")
                h << h.SyncRenderer().a(
                    h.i(class_='ico-btn icon-exit'),
                    class_='leave',
                    title=_(u'Leave "%s"') % self.data.title,
                    onclick=onclick
                ).action(self.leave, comp)
            else:
                # place holder for alignment and for future feature 'request membership'
                h << h.a(h.i(class_='ico-btn icon-user-check'), style='visibility:hidden')
    return h.root


@presentation.render_for(Board, 'redirect')
def render_Board_redirect(self, h, comp, model):
    h << h.script('window.location.href="%s"' % self.data.url)
    return h.root


@presentation.render_for(Board, model="archived_item")
def render_Board_archived_item(self, h, comp, *args):
    with h.li(class_='archived-item'):
        h << h.span(
            h.i(' ', class_=VISIBILITY_ICONS[self.data.visibility]),
            self.data.title,
            href='#', class_="boardItemLabel")

        if security.has_permissions('manage', self):
            with h.div(class_='actions'):
                onclick = 'return confirm("%s")' % _("This board will be destroyed. Are you sure?")
                h << h.SyncRenderer().a(
                    h.i(class_='ico-btn icon-bin'),
                    class_='delete',
                    title=_(u'Delete "%s"') % self.data.title,
                    onclick=onclick
                ).action(self.delete_clicked, comp)
                h << h.a(
                    h.i(class_='ico-btn icon-box-remove'),
                    class_='restore',
                    title=_(u'Restore "%s"') % self.data.title
                ).action(self.restore, comp)
    return h.root


@presentation.render_for(Board, model='members')
def render_Board_members(self, h, comp, *args):
    """Member section view for card

    First "add user" icons,
    Then icon "more user" if necessary
    And at the end member icons
    """
    with h.div(class_='members'):
        if security.has_permissions('Add Users', self):
            h << h.div(self.overlay_add_members, class_='add')
        if len(self.all_members) > self.MAX_SHOWN_MEMBERS:
            h << h.div(self.see_all_members, class_='more wide')
        h << h.div(self.see_all_members_compact, class_='more compact')
        with h.span(class_='wide'):
            for m in self.all_members[:self.MAX_SHOWN_MEMBERS]:
                h << m.on_answer(self.handle_event, comp).render(h, 'overlay')
    return h.root


@presentation.render_for(Board, "members_list_overlay")
def render_Board_members_list_overlay(self, h, comp, *args):
    """Overlay to list all members"""
    h << h.h2(_('All members'))
    with h.form:
        with h.div(class_="members"):
            h << [m.on_answer(self.handle_event, comp).render(h) for m in self.all_members]
    return h.root


def invite_members(board, application_url, emails):
    board.invite_members(emails, application_url)
    return 'reload_boards();'


@presentation.render_for(Board, "add_member_overlay")
def render_Board_add_member_overlay(self, h, comp, *args):
    """Overlay to add member"""
    h << h.h2(_('Invite members'))
    application_url = h.request.application_url
    with h.div(class_="members search"):
        h << self.new_member.on_answer(invite_members, self, application_url)

    return h.root


@presentation.render_for(Icon)
def render_Icon(self, h, comp, *args):
    if self.title is not None:
        h << h.i(class_=self.icon, title=self.title)
        h << self.title
    else:
        h << h.i(class_=self.icon, title=self.title)
    return h.root


@presentation.render_for(Board, 'calendar')
def render_Board_columns(self, h, comp, *args):
    h.head.css_url('js/fullcalendar-2.2.6/fullcalendar.min.css')
    h.head.javascript_url('js/moment.js')
    h.head.javascript_url('js/fullcalendar-2.2.6/fullcalendar.min.js')
    lang = security.get_user().get_locale().language
    if lang != 'en':
        h.head.javascript_url('js/fullcalendar-2.2.6/lang/%s.js' % lang)

    with h.div(id='viewport-wrapper'):
        with h.div(class_='clearfix', id='viewport'):
            h << h.div(id='calendar')
            h << h.script("""YAHOO.kansha.app.create_board_calendar($('#calendar'), %s)""" % ajax.py2js(True, h))
    for column in self.columns:
        if not column().is_archive:
            h << column.render(h, 'calendar')
    return h.root


@presentation.render_for(Board, 'columns')
def render_Board_columns(self, h, comp, *args):
    """Render viewport containing the columns"""
    update_if_version_mismatch = lambda renderer: comp.render(renderer, 'columns') if self.increase_version() else ''

    with h.div(id='viewport-wrapper'):
        with h.div(class_='clearfix', id='viewport'):

            # On cards drag and drop
            action = ajax.Update(action=self.update_card_position, render=update_if_version_mismatch)
            action = '%s;_a;%s=' % (h.add_sessionid_in_url(sep=';'), action._generate_replace(1, h))
            h.head.javascript(h.generate_id(), '''function _send_card_position(data) {
                nagare_getAndEval(%s + YAHOO.lang.JSON.stringify(data));
            }''' % ajax.py2js(action))

            # On columns drag and drop
            action = ajax.Update(action=self.update_column_position, render=update_if_version_mismatch)
            action = '%s;_a;%s=' % (h.add_sessionid_in_url(sep=';'), action._generate_replace(1, h))
            h.head.javascript(h.generate_id(), '''function _send_column_position(data) {
                nagare_getAndEval(%s + YAHOO.lang.JSON.stringify(data));
            }''' % ajax.py2js(action))

            # Create the reload_columns function used when we need to reload
            # the columns from javascript
            reload_board = h.a.action(ajax.Update()).get('onclick').replace('return', "")
            h.head.javascript(h.generate_id(), """function reload_columns(){%s}""" % reload_board)

            increase_version = h.a.action(ajax.Update(render=update_if_version_mismatch))
            h.head.javascript(h.generate_id(), """function increase_version() {%s}""" % increase_version.get('onclick'))

            # Render columns
            visible_cols = len(self.columns) - int(not self.show_archive)
            layout = ''
            if 2 < visible_cols < 6:
                layout = 'list-span-{}'.format(visible_cols)
            elif visible_cols < 3:
                layout = 'list-span-3'

            with h.div(id='lists'):
                if layout:
                    h << {'class': layout}
                h << h.div(' ', id='dnd-frame')
                for column in self.columns:
                    if column().is_archive and not self.show_archive:
                        continue
                    model = 0 if not security.has_permissions('edit', self) else column.model or 'dnd'
                    h << column.on_answer(self.handle_event, comp).render(h, model)
    return h.root


@presentation.render_for(BoardDescription)
def render_BoardDescription(self, h, comp, *args):
    """Render description component in edit mode"""
    h << h.h2(_(u'Edit board description'))
    with h.form(class_='description-form'):
        txt_id, btn_id = h.generate_id(), h.generate_id()
        h << h.label(_(u'Description'), for_=txt_id)
        h << h.textarea(self.description(), id_=txt_id, autofocus=True).action(self.description)
        with h.div(class_='buttons'):
            h << h.button(_('Save'), class_='btn btn-primary', id=btn_id).action(self.commit, comp)
            h << ' '
            h << h.a(_('Cancel'), class_='btn').action(self.cancel, comp)

        h.head.javascript(
            h.generate_id(),
            'YAHOO.kansha.app.addCtrlEnterHandler(%s, %s)' % (
                ajax.py2js(txt_id), ajax.py2js(btn_id)
            )
        )

    return h.root


@presentation.render_for(BoardConfig)
def render_BoardConfig(self, h, comp, *args):
    return comp.render(h.AsyncRenderer(), 'edit')


@presentation.render_for(BoardConfig, model='edit')
def render_BoardConfig_edit(self, h, comp, *args):
    """Render the board configuration panel"""
    h << h.h2(_(u'Board configuration'))
    with h.div(class_='row board-configuration'):
        with h.div(class_='menu'):
            with h.div:
                with h.ul:
                    for id_, item in self.menu.iteritems():
                        with h.li:
                            with h.a.action(lambda id_=id_: self.select(id_)):
                                if id_ == self.selected:
                                    h << {'class': 'active'}
                                h << h.i(class_='icon ' + item.icon)
                                h << h.span(item.label)
        with h.div:
            h << self.content

    h << h.script('YAHOO.kansha.app.hideOverlay();')
    return h.root


@presentation.render_for(BoardLabels, model='menu')
def render_BoardLabels_menu(self, h, comp, *args):
    """Render the link leading to the label configuration"""
    h << h.a(_('Labels')).action(comp.answer)
    return h.root


@presentation.render_for(BoardLabels)
@presentation.render_for(BoardLabels, model='edit')
def render_BoardLabels_edit(self, h, comp, *args):
    """Render the labels configuration panel"""
    with h.div(class_='panel-section'):
        h << h.div(_(u'Card labels'), class_='panel-section-title')
        with h.ul(class_='board-labels clearfix'):
            for title, label in self.labels:
                with h.li:
                    with h.div(class_='label-title'):
                        h << title.render(h.AsyncRenderer())
                    with h.div(class_='label-color'):
                        h << label
    return h.root


@presentation.render_for(BoardWeights, model='menu')
def render_boardweights_menu(self, h, comp, *args):
    """Render the link leading to the weights configuration"""
    h << h.a(_('Weights')).action(comp.answer)
    return h.root


@presentation.render_for(BoardWeights)
@presentation.render_for(BoardWeights, model='edit')
def render_boardweights_edit(self, h, comp, *args):
    """Render the weights configuration panel"""
    with h.div(class_='panel-section'):
        h << h.div(_(u'Weighting cards'), class_='panel-section-title')
        h << h.p(_(u'Activate cards weights'))
        with h.form:
            with h.div(class_='btn-group'):
                action = h.a.action(self.activate_weighting, WEIGHTING_OFF).get('onclick')
                if self.board.total_weight() > 0:
                    action = (
                        "if (confirm(%(message)s)){%(action)s;}return false" %
                        {
                            'action': action,
                            'message': ajax.py2js(
                                _(u'All affected weights will be reset. Are you sure?')
                            ).decode('UTF-8')
                        }
                    )
                h << h.a(
                    _('Disabled'),
                    class_='btn %s' % (
                        'active btn-primary'
                        if self.board.weighting_cards == WEIGHTING_OFF
                        else ''
                    ),
                    onclick=action
                )

                h << h.button(
                    _('Free integer'),
                    class_='btn %s' % (
                        'active btn-primary'
                        if self.board.weighting_cards == WEIGHTING_FREE
                        else ''
                    ),
                    onclick=h.a.action(
                        self.activate_weighting, WEIGHTING_FREE
                    ).get('onclick'),
                    title=_('Card weights can be any integer')
                )

                action = h.a.action(self.activate_weighting, WEIGHTING_LIST).get('onclick')
                if self.board.total_weight() > 0:
                    action = (
                        "if (confirm(%(message)s)){%(action)s;}return false" %
                        {
                            'action': action,
                            'message': ajax.py2js(
                                _(u'All affected weights will be reset. Are you sure?')
                            ).decode('UTF-8')
                        }
                    )
                h << h.button(
                    _('Integer sequence'),
                    class_='btn %s' % (
                        'active btn-primary'
                        if self.board.weighting_cards == WEIGHTING_LIST
                        else ''
                    ),
                    onclick=action,
                    title=_('Choosen within a sequence of integers')
                )

        if self.board.weighting_cards == WEIGHTING_LIST:
            h << h.p(_('Enter a sequence of integers'))
            h << self._weights_editor

    return h.root


@presentation.render_for(WeightsSequenceEditor)
def render_weightssequenceeditor_edit(self, h, comp, model):

    with h.form(class_='weights-form'):
        h << h.input(value=self.weights(), type='text').action(self.weights)
        h << h.button(_('Save'), class_='btn btn-primary').action(self.commit)
        if self.weights.error:
            h << h.div(self.weights.error, class_='nagare-error-message')
        elif self.feedback:
            with h.div(class_='success'):
                h << h.i(class_='icon-checkmark')
                h << self.feedback

    return h.root


@presentation.render_for(BoardProfile)
def render_BoardProfile(self, h, comp, *args):
    """Render the board profile form"""
    if security.has_permissions('manage', self.board):
        with h.div(class_='panel-section'):
            h << h.div(_(u'Content Visibility'), class_='panel-section-title')
            h << h.p(_(u'Choose whether the board is private, public (anyone with the URL can view it ) or shared (public + visible on homepages).'))
            with h.form:
                with h.div(class_='btn-group'):
                    active = 'active btn-primary' if self.board.visibility == BOARD_PRIVATE else ''
                    h << h.button(_('Private'), class_='btn %s' % active).action(
                        lambda: self.board.set_visibility(BOARD_PRIVATE))
                    active = 'active btn-primary' if self.board.visibility == BOARD_PUBLIC else ''
                    h << h.button(_('Public'), class_='btn %s' % active).action(
                        lambda: self.board.set_visibility(BOARD_PUBLIC))
                    active = 'active btn-primary' if self.board.visibility == BOARD_SHARED else ''
                    h << h.button(_('Shared'), class_='btn %s' % active).action(
                        lambda: self.board.set_visibility(BOARD_SHARED))

        with h.div(class_='panel-section'):
            h << h.div(_(u'Comments'), class_='panel-section-title')
            h << h.p(_('Commenting allows members to make short messages on cards. You can enable or disable this feature.'))
            with h.form:
                with h.div(class_='btn-group'):
                    active = 'active btn-primary' if self.board.comments_allowed == COMMENTS_OFF else ''
                    h << h.button(_('Disabled'), class_='btn %s' % active).action(
                        lambda: self.allow_comments(COMMENTS_OFF))
                    active = 'active btn-primary' if self.board.comments_allowed == COMMENTS_MEMBERS else ''
                    h << h.button(_('Members'), class_='btn %s' % active).action(
                        lambda: self.allow_comments(COMMENTS_MEMBERS))
                    kw = {} if self.board.is_open else {
                        "disabled": "disabled"}
                    active = 'active btn-primary' if self.board.comments_allowed == COMMENTS_PUBLIC else ''
                    h << h.button(_('Public'), class_='btn %s' % active, **kw).action(
                        lambda: self.allow_comments(COMMENTS_PUBLIC))

        with h.div(class_='panel-section'):
            h << h.div(_(u'Votes'), class_='panel-section-title')
            h << h.p(_(u'Allow votes'))
            with h.form:
                with h.div(class_='btn-group'):
                    active = 'active btn-primary' if self.board.votes_allowed == VOTES_OFF else ''
                    h << h.button(_('Disabled'), class_='btn %s' %
                                  active).action(lambda: self.allow_votes(VOTES_OFF))
                    active = 'active btn-primary' if self.board.votes_allowed == VOTES_MEMBERS else ''
                    h << h.button(_('Members'), class_='btn %s' % active).action(
                        lambda: self.allow_votes(VOTES_MEMBERS))
                    kw = {} if self.board.is_open else {
                        "disabled": "disabled"}
                    active = 'active btn-primary' if self.board.votes_allowed == VOTES_PUBLIC else ''
                    h << h.button(_('Public'), class_='btn %s' % active,
                                  **kw).action(lambda: self.allow_votes(VOTES_PUBLIC))

        with h.div(class_='panel-section'):
            h << h.div(_(u'Archive'), class_='panel-section-title')
            h << h.p(_(u'View archive column'))
            with h.form:
                with h.div(class_='btn-group'):

                    active = 'active btn-primary' if self.board.show_archive else ''
                    h << h.button(_('Show'), class_='btn %s' % active).action(lambda: self.set_archive(1))
                    active = 'active btn-primary' if not self.board.show_archive else ''
                    h << h.button(_('Hide'), class_='btn %s' % active).action(lambda: self.set_archive(0))

    with h.div(class_='panel-section'):
        h << h.div(_(u'Notifications'), class_='panel-section-title')
        h << h.p(_(u'You will be notified by email of changes made in this board to cards'))
        with h.form:
            with h.div(class_='btn-group'):
                active = 'active btn-primary' if self.notifications_allowed == notifications.NOTIFY_OFF else ''
                h << h.button(_('None'), class_='btn %s' % active).action(self.allow_notifications, notifications.NOTIFY_OFF)
                active = 'active btn-primary' if self.notifications_allowed == notifications.NOTIFY_MINE else ''
                h << h.button(_('Affected to me'), class_='btn %s' % active).action(self.allow_notifications, notifications.NOTIFY_MINE)
                active = 'active btn-primary' if self.notifications_allowed == notifications.NOTIFY_ALL else ''
                h << h.button(_('All'), class_='btn %s' % active).action(self.allow_notifications, notifications.NOTIFY_ALL)
    return h.root


@presentation.render_for(BoardMember)
def render_BoardMember(self, h, comp, *args):
    application_url = h.request.application_url
    if security.has_permissions('manage', self.board):
        return self.user.on_answer(
            lambda action: self.dispatch(action, application_url)
        ).render(h, model='%s' % self.role)
    else:
        return h.div(self.user.render(h), class_='member')


@presentation.render_for(BoardMember, model="overlay")
def render_BoardMember_overlay(self, h, comp, *args):
    application_url = h.request.application_url
    if security.has_permissions('manage', self.board):
        return self.user.on_answer(
            lambda action: self.dispatch(action, application_url)
        ).render(h, model='overlay-%s' % self.role)
    else:
        member = self.user.render(h, "avatar")
        member.attrib.update({'class': 'avatar unselectable'})
        return member


@presentation.render_for(BoardBackground, model='menu')
def render_board_background_menu(self, h, comp, *args):
    return h.a(_('Background image')).action(comp.answer)


@presentation.render_for(BoardBackground)
@presentation.render_for(BoardBackground, model='edit')
def render_board_background_edit(self, h, comp, *args):
    """Render the background configuration panel"""

    with h.div(class_='panel-section'):
        h << h.div(_(u'Background image'), class_='panel-section-title')
        with h.div:
            with h.div:
                v_file = var.Var()
                submit_id = h.generate_id("attach_submit")
                input_id = h.generate_id("attach_input")
                h << h.label((h.i(class_='icon-file-picture'), u' ',
                              _("Choose an image")), class_='btn', for_=input_id)
                with h.form(class_='hidden'):
                    h << h.script(
                        u'''
                function valueChanged(e) {
                    if (YAHOO.kansha.app.checkFileSize(this, %(max_size)s)) {
                        YAHOO.util.Dom.get(%(submit_id)s).click();
                    } else {
                        alert(%(error)s);
                    }
                }

                YAHOO.util.Event.onDOMReady(function() {
                    YAHOO.util.Event.on(%(input_id)s, 'change', valueChanged);
                });''' % {
                            'max_size': ajax.py2js(self.board.background_max_size, h),
                            'input_id': ajax.py2js(input_id, h),
                            'submit_id': ajax.py2js(submit_id, h),
                            'error': ajax.py2js(
                                _(u'Max file size exceeded'), h
                            ).decode('UTF-8')
                        }
                    )
                    h << h.input(id=input_id, class_='hidden', type="file", name="file",
                                 multiple="multiple", maxlength="100",).action(v_file)
                    h << h.input(id=submit_id, class_='hidden', type="submit").action(
                        lambda: self.set_background(v_file()))
                h << ' ' << _('or') << ' '
                h << h.a(_(u'Reset background')).action(self.reset_background)
        with h.p(class_='text-center'):
            h << component.Component(self.board, model='background_image')
        with h.div:
            input_id = h.generate_id()
            submit_id = h.generate_id("image_position_submit")
            h << h.script(u'''YAHOO.util.Event.onDOMReady(function() {
                YAHOO.util.Event.on(%(input_id)s, 'change', function() { YAHOO.util.Dom.get(%(submit_id)s).click(); });
            });''' % {'input_id': ajax.py2js(input_id, h),
                      'submit_id': ajax.py2js(submit_id, h)})
            with h.form:
                h << h.label(_(u'Image position'), for_=input_id)
                with h.select(id_=input_id).action(self.background_position):
                    for value, name in BACKGROUND_POSITIONS:
                        h << h.option(_(name), value=value).selected(self.background_position())
                h << h.input(class_='hidden', id_=submit_id, type="submit").action(self.set_background_position)

    with h.div(class_='panel-section'):
        h << h.div(_(u'Board title color'), class_='panel-section-title')
        with h.div:
            with h.div:
                h << comp.render(h, model='title-color-edit')
                h << ' ' << _('or') << ' '
                h << h.a(_('Reset to default color')).action(self.reset_color)
    return h.root


@presentation.render_for(Board, model='background_image')
def render_board_background_image(self, h, comp, *args):
    fileid = self.data.background_image
    try:
        metadata = self.assets_manager.get_metadata(fileid)
        src = self.assets_manager.get_image_url(fileid, 'medium')
        src += '?r=' + h.generate_id()
        return h.img(title=metadata['filename'], alt=metadata['filename'], src=src)
    except Exception:
        return _(u'No background selected')


@presentation.render_for(BoardBackground, model='title-color-edit')
def render_board_background_title_color_edit(self, h, comp, *args):
    """Edit the label color"""
    # If label changed reload columns
    h << component.Component(overlay.Overlay(lambda r: comp.render(r, model='title-color'),
                                             lambda r: comp.render(r,
                                                                   model='title-color-overlay'),
                                             dynamic=False,
                                             title=_('Change color')))
    return h.root


@presentation.render_for(BoardBackground, model='title-color')
def render_board_background_title_color(self, h, comp, *args):
    style = 'background-color:%s' % (self.board.title_color or u'')
    h << h.span(class_='board-title-color', style=style)
    return h.root


@presentation.render_for(BoardBackground, model='title-color-overlay')
def render_board_background_title_color_overlay(self, h, comp, *args):
    """Color chooser contained in the overlay body"""
    v = var.Var(self.board.title_color)
    i = h.generate_id()
    h << h.div(id=i, class_='label-color-picker clearfix')
    with h.form:
        h << h.input(type='hidden', value=v(), id='%s-hex-value' % i).action(v)
        h << h.button(_('Save'), class_='btn btn-primary').action(
            ajax.Update(action=lambda v=v: self.set_color(v())))
        h << ' '
        h << h.button(_('Cancel'), class_='btn').action(lambda: None)
    h << h.script("YAHOO.kansha.app.addColorPicker(%s)" % ajax.py2js(i))
    return h.root

##### BoardsManager


@presentation.render_for(BoardsManager)
def render_userboards(self, h, comp, *args):
    template = var.Var(u'')
    h.head << h.head.title(self.app_title)

    h.head.css_url('css/themes/home.css?v=2c')
    h.head.css_url('css/themes/%s/home.css?v=2c' % self.theme)

    default_template_i18n = {
        'Empty board': _(u'Empty board'),
        'Basic Kanban': _(u'Basic Kanban')
    }

    with h.div(class_='new-board'):
        with h.form:
            h << h.SyncRenderer().button(_(u'Create'), type='submit', class_='btn btn-primary').action(lambda: self.create_board(template(), comp))
            h << (u' ', _(u'a new'), u' ')

            if len(self.templates) > 1:
                with h.select.action(template):
                    with h.optgroup(label=_(u'Shared templates')):
                        h << [h.option(default_template_i18n.get(tpl, tpl), value=id_) for
                              id_, tpl in self.templates['public']]
                    if self.templates['private']:
                        with h.optgroup(label=_(u'My templates')):
                            h << [h.option(_(tpl), value=id_) for
                                  id_, tpl in self.templates['private']]
            else:
                id_, tpl = self.templates.items()[0]
                template(id_)
                h << tpl

    if self.last_modified_boards:
        h << h.h1(_(u'Last modified boards'))
        with h.ul(class_='board-labels'):
            h << [b.on_answer(self.handle_event).render(h, 'item') for b in self.last_modified_boards]

    h << h.h1(_(u'My boards'))
    if self.my_boards:
        with h.ul(class_='board-labels'):
            h << [b.on_answer(self.handle_event).render(h, 'item') for b in self.my_boards]
    else:
        h << h.p(_(u'Create a board by choosing a template in the menu above, then click on the "Create" button.'))

    if self.guest_boards:
        h << h.h1(_(u'Guest boards'))
        with h.ul(class_='board-labels'):
            h << [b.on_answer(self.handle_event).render(h, 'item') for b in self.guest_boards]

    if self.shared_boards:
        h << h.h1(_(u'Shared boards'))
        with h.ul(class_='board-labels'):
            h << [b.on_answer(self.handle_event).render(h, 'item') for b in self.shared_boards]

    if len(self.archived_boards):
        h << h.h1(_('Archived boards'))

        with h.ul(class_='board-labels'):
            h << [b.on_answer(self.handle_event).render(h, 'archived_item')
                  for b in self.archived_boards]

        with h.form:
            h << h.SyncRenderer().button(
                _('Delete the archived board') if len(self.archived_boards) == 1 else _('Delete the archived boards'),
                class_='delete',
                onclick='return confirm(%s)' % ajax.py2js(
                    _('Deleted boards cannot be restored. Are you sure?')
                ).decode('UTF-8'),
                type='submit'
            ).action(self.purge_archived_boards)

    h << h.script('YAHOO.kansha.app.hideOverlay();'
                  'function reload_boards() { %s; }' % h.AsyncRenderer().a.action(ajax.Update(action=self.load_user_boards, render=0)).get('onclick'))

    return h.root



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from elixir import ManyToOne
from elixir import Field, Date
from elixir import using_options

from kansha.models import Entity


class DataCardDueDate(Entity):
    using_options(tablename='card_due_date')

    due_date = Field(Date, default=None)
    card = ManyToOne('DataCard', ondelete='cascade')

    @classmethod
    def get_by_card(cls, card):
        q = cls.query
        q = q.filter_by(card=card)
        return q.first()



# -*- coding:utf-8 -*-
# --
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
# --

from .comp import BoardCardFilter

from . import view



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from nagare import log
import pkg_resources

from ..assetsmanager import AssetsManager


class DummyAssetsManager(AssetsManager):

    def __init__(self):
        super(DummyAssetsManager, self).__init__('', None)

    def save(self, data, file_id=None, metadata={}):
        log.debug("Save Image")
        log.debug("%s" % metadata)
        return 'mock_id'

    def load(self, file_id):
        log.debug("Load Image")
        package = pkg_resources.Requirement.parse('kansha')
        fname = pkg_resources.resource_filename(package, 'kansha/services/dummyassetsmanager/tie.jpg')
        with open(fname, 'r') as f:
            data = f.read()
        return data, {}

    def update_metadata(self, file_id, metadata):
        pass

    def get_metadata(self, file_id):
        pass



"""Data in extensions

Revision ID: f058ce7ee0c
Revises: 2b0edcfa57b4
Create Date: 2016-01-07 13:41:19.534150

"""

import time
from datetime import date

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'f058ce7ee0c'
down_revision = '361f9cbae3fc'


def upgrade():
    bind = op.get_bind()
    is_sqlite = bind.engine.name == 'sqlite'

    op.create_table('card_description',
                    sa.Column('id', sa.Integer, primary_key=True, autoincrement=True),
                    sa.Column('description', sa.UnicodeText, default=lambda: u''),
                    sa.Column('card_id', sa.Integer, sa.ForeignKey('card.id', ondelete='CASCADE')))
    op.create_table('card_due_date',
                    sa.Column('id', sa.Integer, primary_key=True, autoincrement=True),
                    sa.Column('due_date', sa.Date),
                    sa.Column('card_id', sa.Integer, sa.ForeignKey('card.id', ondelete='CASCADE')))
    op.create_table('card_weight',
                    sa.Column('id', sa.Integer, primary_key=True, autoincrement=True),
                    sa.Column('weight', sa.Unicode(255), default=lambda: u''),
                    sa.Column('card_id', sa.Integer, sa.ForeignKey('card.id', ondelete='CASCADE')))
    descriptions = []
    due_dates = []
    weights = []
    descriptions_table = sa.Table(
        'card_description',
        sa.MetaData(),
        sa.Column('id', sa.Integer, primary_key=True, autoincrement=True),
        sa.Column('description', sa.UnicodeText, default=lambda: u''),
        sa.Column('card_id', sa.Integer, sa.ForeignKey('card.id', ondelete='CASCADE'))
    )
    due_dates_table = sa.Table(
        'card_due_date',
        sa.MetaData(),
        sa.Column('id', sa.Integer, primary_key=True, autoincrement=True),
        sa.Column('due_date', sa.Date),
        sa.Column('card_id', sa.Integer, sa.ForeignKey('card.id', ondelete='CASCADE'))
    )
    weights_table = sa.Table(
        'card_weight',
        sa.MetaData(),
        sa.Column('id', sa.Integer, primary_key=True, autoincrement=True),
        sa.Column('weight', sa.Unicode(255), default=lambda: u''),
        sa.Column('card_id', sa.Integer, sa.ForeignKey('card.id', ondelete='CASCADE'))
    )

    select = sa.text('SELECT id, description, due_date, weight FROM card')
    for card_id, description, due_date, weight in bind.execute(select):
        if due_date is not None and is_sqlite:
            due_date = time.strptime(due_date, '%Y-%m-%d')
            due_date = date(due_date.tm_year, due_date.tm_mon, due_date.tm_mday)

        descriptions.append({'card_id': card_id,
                             'description': description})
        due_dates.append({'card_id': card_id,
                          'due_date': due_date})
        weights.append({'card_id': card_id,
                        'weight': weight})

    op.bulk_insert(descriptions_table, descriptions)
    op.bulk_insert(due_dates_table, due_dates)
    op.bulk_insert(weights_table, weights)

    if not is_sqlite: # SQLite doesn't support column dropping
        op.drop_column('card', 'description')
        op.drop_column('card', 'due_date')
        op.drop_column('card', 'weight')


def downgrade():
    op.add_column('card', sa.Column('description', sa.UnicodeText, default=u''))
    op.drop_table('card_description')

    op.add_column('card', sa.Column('due_date', sa.Date))
    op.drop_table('card_due_date')

    op.add_column('card', sa.Column('weight', sa.Unicode(255)))
    op.drop_table('card_weight')



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from nagare import presentation, ajax
from nagare.i18n import _


class Overlay(object):

    """Overlay component
    """

    def __init__(self, text_factory, content_factory, title=None, dynamic=True, cls=None, centered=False):
        """Initialization

        In:
            - ``text_factory`` -- a callable generating the overlay title
            - ``content_factory`` -- a callable generating the overlay content
            - ``title`` -- if given, the text will be used in the overlay title when showed
            - ``dynamic`` -- if True, the content of the overlay will be filled using AJAX.
            - ``cls`` -- an additional class name
        """
        self.text = text_factory
        self.content = content_factory
        self.title = title
        self.dynamic = dynamic
        self.cls = cls
        self.centered = centered

    def render_a(self, h, link_id):
        h << h.a(self.text(h), href='#', id_=link_id, title=self.title or '')


@presentation.render_for(Overlay)
def render(self, h, comp, *args):
    """Render the overlay"""
    link_id = h.generate_id()
    overlay_id = h.generate_id()
    body_id = h.generate_id('BODY_')

    self.render_a(h, link_id)

    # TODO: find a cleaner solution
    if self.dynamic:
        # Load overlay body dynamically
        load = ajax.js(
            "YAHOO.util.Dom.get(%(body_id)s).innerHTML = '';"
            "%(load_body)s" %
            {
                'body_id': ajax.py2js(body_id),
                'load_body': h.a.action(
                    ajax.Update(render=self.content, component_to_update=body_id)
                ).get('onclick').replace('return', '')
            }
        )
    else:
        # The overlay body is already present in the DOM tree
        load = ajax.js('')

    js = ajax.js(
        "%(load)s;"
        "YAHOO.kansha.app.showOverlay(%(overlay_id)s, %(link_id)s, %(centered)s)" %
        {
            'load': ajax.py2js(load),
            'link_id': ajax.py2js(link_id),
            'overlay_id': ajax.py2js(overlay_id),
            'centered': ajax.py2js(self.centered, h)
        }
    )

    h.head.javascript(
        h.generate_id(),
        "YAHOO.util.Event.addListener(%(link_id)s, 'click', "
        "function(){%(js)s;}"
        ")" % {'link_id': ajax.py2js(link_id), "js": ajax.py2js(js)}
    )

    cls = [u'overlay']
    if self.cls:
        cls.append(self.cls)
    with h.div(class_=u' '.join(cls), id=overlay_id, onclick="YAHOO.kansha.app.stopEvent()"):
        if not self.centered:
            h << h.span(class_='overlay_arrow')
        with h.div(class_='overlay-header'):
            with h.div(class_='overlay-title'):
                h << h.div(self.title or self.text(h))
        h << h.div(' ' if self.dynamic else self.content(h), class_='overlay-body', id=body_id)

    return h.root



#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from nagare import database
from sqlalchemy import func
import sqlalchemy as sa
from sqlalchemy.ext.associationproxy import association_proxy
from elixir import (ManyToOne, ManyToMany, OneToMany, using_options, Field, Boolean, Integer,
                    using_table_options)

from kansha.models import Entity
from kansha.card.models import DataCard
from kansha.user.models import DataUser

# Levels
NOTIFY_OFF = 0
NOTIFY_MINE = 1
NOTIFY_ALL = 2


class DataCardMembership(Entity):
    using_options(tablename='membership_cards__card')
    membership = ManyToOne('DataMembership', ondelete='cascade', required=True)
    card = ManyToOne('DataCard', ondelete='cascade', required=True)
    using_table_options(sa.UniqueConstraint('membership_id', 'card_id', name='card_membership_ix'))

    @classmethod
    def purge(cls, card):
        for member in cls.query.filter_by(card=card):
            member.delete()


class DataMembership(Entity):
    using_options(tablename='membership')
    board = ManyToOne('DataBoard', ondelete='cascade', required=True)
    user = ManyToOne(DataUser, ondelete='cascade', required=True)
    card_memberships = OneToMany(DataCardMembership, cascade='delete, delete-orphan')
    # provisional, for notifications until they are refactored
    cards = association_proxy('card_memberships', 'card')
    manager = Field(Boolean, default=False, nullable=False)
    notify = Field(Integer, default=NOTIFY_MINE)
    using_table_options(
        sa.UniqueConstraint('board_id', 'user_username', 'user_source', name='membership_ix'))

    @classmethod
    def get_for_card(cls, card):
        return cls.query.join(DataCardMembership).filter(DataCardMembership.card == card)

    @classmethod
    def search(cls, board, user):
        return cls.get_by(board=board, user=user)  # at most one

    @classmethod
    def subscribers(cls):
        return cls.query.filter(sa.or_(DataMembership.notify == NOTIFY_ALL,
                                       DataMembership.notify == NOTIFY_MINE))

    @staticmethod
    def favorites_for(card):
        query = database.session.query(DataUser)
        query = query.join(DataMembership)
        # In the future, cards will be linked to boards directly, so demeter won't be hurt anymore
        query = query.filter(DataMembership.board == card.column.board)
        query = query.outerjoin(DataCardMembership)
        query = query.group_by(DataUser)
        query = query.order_by(func.count(DataUser.username).desc())
        return query

    @classmethod
    def add_card_members_from_emails(cls, card, emails):
        """Provisional: will take memberships instead of emails."""
        query = cls.query.join(DataUser).filter(sa.or_(DataUser.email.in_(emails),
                                                       DataUser.email_to_confirm.in_(emails)),
                                                DataMembership.board == card.column.board)
        memberships = query.all()
        for membership in memberships:
            membership.card_memberships.append(DataCardMembership(card=card))
        database.session.flush()
        return memberships

    @classmethod
    def remove_card_member(cls, card, username):
        """Provisional: will take a membership instead of username."""
        membership = cls.query.join(DataUser).filter(
            DataUser.username == username,
            cls.board == card.column.board).first()
        if membership:
            card_membership = DataCardMembership.get_by(card=card, membership=membership)
            if card_membership:
                card_membership.delete()

    @classmethod
    def add_member(cls, board, user, manager=False):
        membership = cls(board=board, user=user, manager=manager)
        database.session.add(membership)
        database.session.flush()
        return membership

    @classmethod
    def remove_member(cls, board, user):
        membership = cls.get_by(board=board, user=user)
        if membership:
            membership.delete()
            database.session.flush()

    @classmethod
    def has_member(cls, board, user, manager=False):
        membership = cls.get_by(board=board, user=user)
        return (bool(membership) and membership.manager) if manager else bool(membership)

    @classmethod
    def delete_members(cls, board):
        cls.query.filter_by(board=board).delete(synchronize_session=False)

    @classmethod
    def change_role(cls, board, user, manager):
        ms = cls.get_by(board=board, user=user)
        if ms:
            ms.manager = manager
            database.session.flush()



"""Rename board.archive

Revision ID: 361f9cbae3fc
Revises: 2b0edcfa57b4
Create Date: 2015-12-18 17:00:10.034124

"""

import sqlalchemy as sa
from alembic import op


# revision identifiers, used by Alembic.
revision = '361f9cbae3fc'
down_revision = '2b0edcfa57b4'


def upgrade():
    with op.batch_alter_table('board') as batch_op:
        batch_op.alter_column('archive', new_column_name='show_archive')


def downgrade():
    with op.batch_alter_table('board') as batch_op:
        batch_op.alter_column('show_archive', new_column_name='archive')



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from nagare import presentation, var, database, security
from nagare.i18n import _
from nagare.admin.reference import load_object

from ...user import usermanager
from kansha.services.authentication_repository import Authentication


class Login(Authentication):

    alt_title = None

    CONFIG_SPEC = {
        'activated': 'boolean(default=False)',
        'host': 'string(default="localhost")',
        'port': 'integer(default=389)',
        'users_base_dn': 'string(default="")',
        'schema': 'string(default="kansha.authentication.ldap.ldap_auth:NngLDAPAuth")'
    }

    def __init__(self, app_title, app_banner, theme, assets_manager_service):
        cls = load_object(self.config['schema'])[0]
        self.ldap_engine = cls(self.config)
        self.error_message = ''
        self.assets_manager = assets_manager_service

    def connect(self, uid, passwd, comp):
        if (uid != '' and passwd != '' and
                self.ldap_engine.check_password(uid, passwd)):
            data_user = usermanager.UserManager.get_by_username(uid)
            profile = self.ldap_engine.get_profile(uid, passwd)
            # if user exists update data
            if profile['picture']:
                self.assets_manager.save(profile['picture'], uid,
                                         {'filename': uid})
                picture = self.assets_manager.get_image_url(uid, 'thumb')
            else:
                picture = None
            if not data_user:
                data_user = usermanager.UserManager().create_user(uid, None,
                                                                  profile['name'], profile['email'], source='ldap',
                                                                  picture=picture)
            data_user.update(profile['name'], profile['email'],
                             picture=picture)
            database.session.flush()
            u = usermanager.UserManager.get_app_user(uid, data=data_user)
            security.set_user(u)
            comp.answer(u)
        else:
            self.error_message = _('Login failed')


@presentation.render_for(Login)
def render(self, h, comp, *args):
    with h.div(class_='login LDAPLogin'):
        with h.form:
            # if self.error_message:
            #     h << h.br << h.div(self.error_message, class_="error")
            uid = var.Var()
            h << h.input(type='text', name='__ac_name',
                         id='ldap_username', placeholder=_('Enter username')).action(uid)
            passwd = var.Var()
            h << h.input(
                type='password', name='__ac_password',
                id='ldap_password', placeholder=_('Enter password')).action(passwd)
            with h.div(class_='actions'):
                h << h.input(type='submit', value=_(u'Sign in with LDAP'),
                             class_='btn btn-primary'
                             ).action(lambda: self.connect(uid(), passwd(), comp))

    return h.root



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

import smtplib
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.Utils import COMMASPACE, formatdate

from nagare import log

from .services_repository import Service


class MailSender(Service):
    '''
    Mail sender service.

    API.
    A mail sender service must provide a send method:
    '''

    LOAD_PRIORITY = 10
    CONFIG_SPEC = {
        'activated': 'boolean(default=True)',
        'host': 'string(default="127.0.0.1")',
        'port': 'integer(default=25)',
        'default_sender': 'string(default="noreply@email.com")'
    }

    def __init__(self, config_filename, error, host, port, default_sender, activated):
        super(MailSender, self).__init__(config_filename, error)
        self.host = host
        self.port = port
        self.default_sender = default_sender
        self.activated = activated
        if self.activated:
            log.debug(
                'The mail service will connect to %s on port %s' %
                (self.host, self.port)
            )
        else:
            log.warning('The mail service will drop all messages!')

    def _smtp_send(self, from_, to, contents):
        smtp = smtplib.SMTP(self.host, self.port)
        try:
            smtp.sendmail(from_, to, contents)
        except Exception as e:
            log.exception(e)
        finally:
            smtp.close()

    def send(self, subject, to, content, html_content=None, from_='', cc=[], bcc=[],
             type='plain', mpart_type='alternative'):
        """Sends an email

        In:
         - ``subject`` -- email subject
         - ``to`` -- list of recipients' emails
         - ``content`` --  email content
         - ``from_`` -- email sender adress
         - ``cc`` --  list of CC emails
         - ``bcc`` -- list of BCC emails
         - ``type`` --  email type ('plain' or 'html')
         - ``mpart_type`` -- email part type

        """
        from_ = from_ if from_ else self.default_sender
        # create the message envelop
        msg = MIMEMultipart(mpart_type)
        msg['Subject'] = subject
        msg['Date'] = formatdate(localtime=True)
        msg['From'] = from_
        msg['To'] = COMMASPACE.join(to)

        if cc:
            msg['Cc'] = COMMASPACE.join(cc)

        # attach the mail content
        charset = 'us-ascii'
        if isinstance(content, unicode):
            content = content.encode('UTF-8')
            charset = 'UTF-8'
        msg.attach(MIMEText(content, type, charset))
        if html_content:
            msg.attach(MIMEText(html_content, 'html', charset))

        # log
        log.info('%s mail:\n  subject=%s\n  from=%s\n  to=%s\n  cc=%s\n  bcc=%s',
                 'sending' if self.activated else 'ignoring', subject, from_, to, cc, bcc)
        log.debug('Mail content:\n' + content)

        # post the email to the SMTP server
        if self.activated:
            self._smtp_send(from_, to + cc + bcc, msg.as_string())


class DummyMailSender(MailSender):
    '''For use in unit tests.'''

    def __init__(self):
        super(DummyMailSender, self).__init__(
            '',
            None,
            host='localhost',
            port=25,
            activated=False,
            default_sender='noreply@test.test'
        )



#--
# Copyright (c) 2008-2013 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

#--
# Copyright (c) 2012-2015 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--


"""
Create and (re)build the search index for cards.
It is safe to run it anytime.
Registered as a nagare-admin command.
Usage :
nagare-admin create-index <app name | config file>
"""

import pkg_resources

from nagare import database
from nagare.admin import util, command

from kansha.card.comp import Card
from kansha.column.comp import Column
from kansha.card.models import DataCard
from kansha.services.actionlog import DummyActionLog


# FIXME: too low level
def rebuild_index(app):
    # init index
    action_log = DummyActionLog()
    app.search_engine.create_collection([Card.schema])
    for card in DataCard.query:
        board_id = card.column.board.id
        card = app._services(Card, card.id, app.card_extensions, action_log, lambda x: True, data=card)
        card.add_to_index(app.search_engine, board_id)
    app.search_engine.commit()


class ReIndex(command.Command):

    desc = '(Re-)Create index for the application.'

    @staticmethod
    def set_options(optparser):
        optparser.usage += ' [application]'

    @staticmethod
    def run(parser, options, args):

        try:
            application = args[0]
        except IndexError:
            application = 'kansha'

        (cfgfile, app, dist, conf) = util.read_application(application,
                                                           parser.error)
        requirement = (
            None if not dist
            else pkg_resources.Requirement.parse(dist.project_name)
        )
        data_path = (
            None if not requirement
            else pkg_resources.resource_filename(requirement, '/data')
        )

        (active_app, databases) = util.activate_WSGIApp(
            app, cfgfile, conf, parser.error, data_path=data_path)
        for (database_settings, populate) in databases:
            database.set_metadata(*database_settings)
        if active_app:
            rebuild_index(active_app)



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from nagare import ajax, serializer
import peak


class ActionViewToJs(ajax.ViewToJs):

    def __init__(self, output):
        self.output = output


@peak.rules.when(ajax.serialize_body, (ActionViewToJs,))
def serialize_body(view_to_js, content_type, doctype):
    return view_to_js.output


@peak.rules.when(serializer.serialize, (ActionViewToJs,))
def serialize(self, content_type, doctype, declaration):
    return ('text/plain', self.output)


class Action(ajax.Update):

    def __init__(self, action):
        # The ``action`` function will be called on the server
        # with a renderer parameter if ``with_renderer=True`
        # else without any parameter.
        #
        # It can return a javascript code which will be executed
        # on the client.
        super(Action, self).__init__(render=action, component_to_update='')

    @classmethod
    def _generate_response(cls, render, args, js, component_to_update, r):
        return ActionViewToJs(cls.generate_javascript(render, args))

    @classmethod
    def generate_javascript(cls, render, args):
        return render(*args)


class Actions(Action):

    def __init__(self, *actions):
        super(Actions, self).__init__(lambda: actions, True)

    def generate_javascript(self, h):
        return ';'.join(action.generate_javascript(h) for action in self.render())



#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""last_login stamp

Revision ID: 4fb34b202c5f
Revises: 6cb3c668d65
Create Date: 2015-05-18 19:00:56.564336

"""

import datetime

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = '4fb34b202c5f'
down_revision = '6cb3c668d65'


def upgrade():
    last_login = sa.Column('last_login', sa.DateTime)
    op.add_column('user', last_login)

    # last_login mustn't be empty for already existing users
    user = sa.table(
        'user',
        last_login,
        sa.Column('email_to_confirm', sa.Unicode(255))
    )
    op.execute(
        user.update().where(
            user.c.email_to_confirm == None  # noqa
        ).values({'last_login': datetime.datetime.now()})
    )


def downgrade():
    op.drop_column('user', 'last_login')



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

import datetime

from nagare.database import session

from sqlalchemy import func
from elixir import ManyToOne, Field, Unicode, DateTime, using_options

from kansha.models import Entity


class DataAsset(Entity):

    using_options(tablename='asset')
    filename = Field(Unicode(255), primary_key=True)
    creation_date = Field(DateTime)
    author = ManyToOne('DataUser')
    card = ManyToOne('DataCard')
    cover = ManyToOne('DataCard')

    @classmethod
    def get_all(cls, card):
        return cls.query.filter_by(card=card)

    @classmethod
    def count_for(cls, card):
        return cls.get_all(card).with_entities(func.count()).scalar()

    @classmethod
    def add(cls, filename, card, author):
        asset = cls(filename=filename, card=card, author=author,
                   creation_date=datetime.datetime.utcnow())
        session.add(asset)
        session.flush()
        return asset

    @classmethod
    def remove(cls, card, filename):
        file = cls.get_by_filename(filename)
        file.delete()

    @classmethod
    def remove_all(cls, card):
        q = cls.query
        q = q.filter_by(card=card)
        q.delete()

    @classmethod
    def get_cover(cls, card):
        q = cls.query
        q = q.filter_by(cover=card)
        return q.first()

    @classmethod
    def set_cover(cls, card, asset):
        cls.remove_cover(card)
        asset.cover = card

    @classmethod
    def remove_cover(cls, card):
        asset = cls.get_cover(card)
        if asset is not None:
            asset.cover = None

    @classmethod
    def get_by_filename(cls, filename):
        return cls.get(filename)



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from nagare import security

from kansha.cardextension.tests import CardExtensionTestCase

from .comp import Comments


class CommentsTest(CardExtensionTestCase):

    extension_name = 'comments'
    extension_class = Comments

    def test_add_delete(self):
        self.assertEqual(len(self.extension.comments), 0)
        self.extension.add(u'test')
        self.assertEqual(len(self.extension.comments), 1)
        comment = self.extension.comments[0]()
        self.assertEqual(comment.data.comment, u'test')
        comment.set_comment(u'test2')
        self.assertEqual(comment.data.comment, u'test2')
        self.extension.delete_comment(self.extension.comments[0])

    def test_comment_label(self):
        self.extension.add(u'test')
        label = self.extension.comments[0]().comment_label()
        self.assertEqual(label.text, u'test')
        label.change_text(u'test2')
        self.assertEqual(label.text, u'test2')
        self.assertTrue(label.is_author(security.get_user()))

    def test_update_document(self):
        doc = self.card.schema(docid=None)
        self.extension.add(u'Comment 1')
        self.extension.add(u'Comment 2')
        self.extension.add(u'Comment 3')
        self.extension.update_document(doc)
        self.assertEqual(doc.comments, u'''Comment 3
Comment 2
Comment 1''')




# -*- coding:utf-8 -*-
# --
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
# --

import json
from datetime import datetime, timedelta

from sqlalchemy.types import TypeDecorator

from elixir import ManyToOne
from elixir import using_options
from elixir import Field, Unicode, UnicodeText, DateTime

from nagare import database

from kansha.models import Entity

from .messages import render_event


# Models


class JSONType(TypeDecorator):
    impl = UnicodeText

    def process_bind_param(self, value, dialect):
        if value is not None:
            value = unicode(json.dumps(value))
        return value

    def process_result_value(self, value, dialect):
        if value is not None:
            value = json.loads(value)
        return value


class DataHistory(Entity):
    using_options(tablename='history', order_by='-when')

    when = Field(DateTime)
    action = Field(Unicode(255))
    data = Field(JSONType)

    board = ManyToOne('DataBoard', ondelete='cascade')
    card = ManyToOne('DataCard', ondelete='cascade', required=True)
    user = ManyToOne('DataUser', ondelete='cascade', required=True)

    def to_string(self):
        data = self.data.copy()
        data['author'] = self.user.fullname or self.user.username
        return render_event(self.action, data)

    @classmethod
    def add_history(cls, board, card, user, action, data):
        data.update(action=action)
        when = datetime.utcnow()
        data = cls(
            when=when, action=action, board=board, card=card, user=user, data=data)
        database.session.flush()

    @classmethod
    def get_events(cls, board, hours=None):
        '''board to None means "everything".'''
        since = datetime.utcnow() - timedelta(hours=hours)
        q = cls.query
        if board:
            q = q.filter_by(board=board)
        q = q.filter(cls.when >= since)
        q = q.order_by(cls.board_id, cls.action, cls.when)
        return q.all()

    @classmethod
    def get_history(cls, board, cardid=None, username=None):
        q = cls.query
        q = q.filter_by(board=board)
        if cardid:
            q = q.filter(cls.card.has(id=cardid))
        if username:
            q = q.filter(cls.user.has(username=username))
        return q

    @classmethod
    def get_last_activity(cls, board):
        q = database.session.query(cls.when)
        q = q.filter(cls.board == board)
        q = q.order_by(cls.when.desc())
        q = q.limit(1)
        return q.scalar()

    @classmethod
    def purge(cls, card):
        q = database.session.query(cls).filter(cls.card == card)
        for log in q:
            log.delete()



"""weights are numbers

Revision ID: 443501b69d16
Revises: 55f89221fc55
Create Date: 2017-04-03 18:28:50.108072

"""

import sqlalchemy as sa
from alembic import op


# revision identifiers, used by Alembic.
revision = '443501b69d16'
down_revision = '55f89221fc55'


def upgrade():
    bind = op.get_bind()
    op.create_table('board_weight_config',
                    sa.Column('id', sa.Integer, primary_key=True, autoincrement=True),
                    sa.Column('weights', sa.Unicode(255), default=u'0, 1, 2, 3, 5, 8, 13'),
                    sa.Column('weighting_cards', sa.Integer, default=0),
                    sa.Column('board_id', sa.Integer, sa.ForeignKey('board.id', ondelete='CASCADE')))
    # collect existing weights
    weights = {}
    select = sa.text('SELECT card_id, weight FROM card_weight')
    for card_id, weight in bind.execute(select):
        weights[card_id] = int(weight or 0)
    op.drop_column('card_weight', 'weight')

    # data migration
    op.add_column('card_weight', sa.Column('weight', sa.Integer))
    weights_table = sa.Table(
        'card_weight',
        sa.MetaData(),
        sa.Column('id', sa.Integer, primary_key=True, autoincrement=True),
        sa.Column('weight', sa.Integer, default=0),
        sa.Column('card_id', sa.Integer, sa.ForeignKey('card.id', ondelete='CASCADE'))
    )
    for card_id, weight in weights.iteritems():
        bind.execute(
            weights_table.update().where(
                weights_table.c.card_id == card_id
            ).values(
                weight=weight
            )
        )
    board_weight_config = sa.Table(
        'board_weight_config',
        sa.MetaData(),
        sa.Column('id', sa.Integer, primary_key=True, autoincrement=True),
        sa.Column('weights', sa.Unicode(255), default=u'0, 1, 2, 3, 5, 8, 13'),
        sa.Column('weighting_cards', sa.Integer, default=0),
        sa.Column('board_id', sa.Integer, sa.ForeignKey('board.id', ondelete='CASCADE')))

    board = sa.Table(
        'board',
        sa.MetaData(),
        sa.Column('id', sa.Integer, primary_key=True, autoincrement=True),
        sa.Column('weights', sa.Unicode(255), default=u'0, 1, 2, 3, 5, 8, 13'),
        sa.Column('weighting_cards', sa.Integer, default=0),
    )
    conf = [
        dict(weights=board_conf.weights or u'0, 1, 2, 3, 5, 8, 13', weighting_cards=board_conf.weighting_cards, board_id=board_conf.id)
        for board_conf in bind.execute(board.select())
    ]
    op.bulk_insert(board_weight_config, conf)
    # cleaning
    op.drop_column('board', 'weights')
    op.drop_column('board', 'weighting_cards')


def downgrade():
    print "there is no way back!"



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

import random

from nagare import component, var, i18n

from kansha.toolbox import overlay
from kansha.board import excel_export
from kansha.services.search import schema
from kansha.cardextension import CardExtension

from .models import DataLabel


class Label(object):

    """Label component

    6 labels by board
    A label contains tuple (color, title)
    """

    def __init__(self, data):
        """Initialization

        In:
          - ``data`` -- the label's database object
        """
        self.id = data.id
        self._data = data

    @property
    def data(self):
        """Return the label from the database

        Return:
         - the DataLabel instance
        """
        if self._data is None:
            self._data = DataLabel.get(self.id)
        return self._data

    def __getstate__(self):
        self._data = None
        return self.__dict__

    @property
    def index(self):
        return self.data.index

    def get_color(self):
        return self.data.color

    def set_color(self, v):
        """Change the color of the label

        In:
          - ``v`` -- the color of the label as hex string (ex. '#CECECE')
        """
        self.data.color = v

    def get_title(self):
        return self.data.title

    def set_title(self, title):
        """Set title

        In:
            - ``title`` -- new title
        """
        self.data.title = title

    def add(self, card):
        self.data.add(card.data)

    def remove(self, card):
        self.data.remove(card.data)

    def __eq__(self, other):
        return self.id == other.id


class CardLabels(CardExtension):

    """Card labels component

    This component represents all labels associated with a card
    """

    LOAD_PRIORITY = 10

    def __init__(self, card, action_log, configurator):
        """Initialization

        In:
          - ``card`` -- the card object (Card instance)
        """
        super(CardLabels, self).__init__(card, action_log, configurator)
        self.comp_id = str(random.randint(10000, 100000))
        self.labels = [Label(label) for label in self.data]

    @staticmethod
    def get_schema_def():
        return schema.Text('labels')

    def update_document(self, document):
        document.labels = u' '.join(label.get_title() for label in self.labels)

    @property
    def data(self):
        return DataLabel.get_by_card(self.card.data)

    def update(self, other):

        active_labels = set(label.get_title() for label in other.labels)
        for label in self.get_available_labels():
            if label.get_title() in active_labels:
                self.activate(label)

    @property
    def colors(self):
        """Returns the colors of the labels associated with the card
        """
        return (l.get_color() for l in self.labels)

    def get_available_labels(self):
        """Returns the labels available in the card's board
        """
        return self.configurator.labels if self.configurator else []

    def activate(self, label):
        """Adds/removes a label to the current card.

        If the label is already associated with the card,
        remove it. Else add it.

        """
        if label in self.labels:
            label.remove(self.card)
            self.labels.remove(label)
        else:
            label.add(self.card)
            self.labels.append(label)
            self.labels.sort(key=lambda l: l.index)

    def delete(self):
        for label in self.labels:
            label.remove(self.card)
        self.labels = []


@excel_export.get_extension_title_for(CardLabels)
def get_extension_title_CardDescription(card_extension):
    return i18n._(u'Labels')


@excel_export.write_extension_data_for(CardLabels)
def write_extension_data_CardDescription(self, sheet, row, col, style):
    text = u', '.join(label.get_title() for label in self.labels)
    sheet.write(row, col, text, style)



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

import logging
import unittest

from nagare import database
from elixir import metadata as __metadata__

from kansha import helpers
from kansha.authentication.database import forms

database.set_metadata(__metadata__, 'sqlite:///:memory:', False, {})


class TokenTest(unittest.TestCase):

    def setUp(self):
        helpers.setup_db(__metadata__)
        self.token_generator = forms.TokenGenerator(u'test_username',
                                                    u'reset_password')

    def tearDown(self):
        helpers.teardown_db(__metadata__)

    def test_create_token(self):
        token = self.token_generator.create_token()

        logging.debug("%s" % token)

        self.assertEqual(token.username, u'test_username')
        self.assertEqual(token.action, u'reset_password')
        self.assertEqual(token.board, None)



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

import unittest

from nagare import database
from elixir import metadata as __metadata__

from kansha import helpers
from kansha.board import boardsmanager
from kansha.board.models import DataBoard
from kansha.board import comp as board_module


database.set_metadata(__metadata__, 'sqlite:///:memory:', False, {})


# FIXME: dirty tests; rewrite them all on component API.

class BoardTest(unittest.TestCase):

    def setUp(self):
        helpers.setup_db(__metadata__)
        self.boards_manager = helpers.get_boards_manager()

    def tearDown(self):
        helpers.teardown_db(__metadata__)

    def test_add_board(self):
        """Create a new board"""
        helpers.set_dummy_context()
        self.assertEqual(DataBoard.query.count(), 0)
        helpers.create_board()
        self.assertEqual(DataBoard.query.filter_by(is_template=False).count(), 1)

    def test_add_column_ok(self):
        """Add a column to a board"""
        helpers.set_dummy_context()
        board = helpers.create_board()
        self.assertIsNotNone(board.archive_column)
        self.assertEqual(board.count_columns(), 4)
        board.create_column(0, helpers.word())
        self.assertEqual(board.count_columns(), 5)

    def test_add_column_ko(self):
        """Add a column with empty title to a board"""
        helpers.set_dummy_context()
        board = helpers.create_board()
        self.assertEqual(board.count_columns(), 4)
        self.assertFalse(board.create_column(0, ''))

    def test_delete_column(self):
        """Delete column from a board"""
        helpers.set_dummy_context()
        user = helpers.create_user()
        helpers.set_context(user)
        board = helpers.create_board()
        self.assertIsNotNone(board.archive_column)
        self.assertEqual(board.count_columns(), 4)
        column = board.columns[0]
        board.delete_column(column)
        self.assertEqual(board.count_columns(), 3)

    def test_set_visibility_1(self):
        """Test set visibility method 1

        Initial State:
         - board:private
         - allow_comment: off
         - allow_votes: members

        End state
         - board:public
         - allow_comment: off
         - allow_votes: members
        """
        helpers.set_dummy_context()
        board = helpers.create_board()
        board.data.visibility = 0
        board.data.comments_allowed = 0
        board.data.votes_allowed = 1

        board.set_visibility(board_module.BOARD_PUBLIC)

        self.assertEqual(board.data.visibility, 1)
        self.assertEqual(board.data.comments_allowed, 0)
        self.assertEqual(board.data.votes_allowed, 1)

    def test_set_visibility_2(self):
        """Test set visibility method 2

        Initial State:
         - board:public
         - allow_comment: public
         - allow_votes: public

        End state
         - board:private
         - allow_comment: members
         - allow_votes: members
        """
        helpers.set_dummy_context()
        board = helpers.create_board()
        board.data.visibility = 1
        board.data.comments_allowed = 2
        board.data.votes_allowed = 2

        board.set_visibility(board_module.BOARD_PRIVATE)

        self.assertEqual(board.data.visibility, 0)
        self.assertEqual(board.data.comments_allowed, 1)
        self.assertEqual(board.data.votes_allowed, 1)

    def test_set_visibility_3(self):
        """Test set visibility method 3

        Initial State:
         - board:public
         - allow_comment: members
         - allow_votes: off

        End state
         - board:private
         - allow_comment: members
         - allow_votes: off
        """
        helpers.set_dummy_context()
        board = helpers.create_board()
        board.data.visibility = 1
        board.data.comments_allowed = 1
        board.data.votes_allowed = 0

        board.set_visibility(board_module.BOARD_PRIVATE)

        self.assertEqual(board.data.visibility, 0)
        self.assertEqual(board.data.comments_allowed, 1)
        self.assertEqual(board.data.votes_allowed, 0)

    def test_save_as_template(self):
        title = helpers.word()
        description = helpers.word()
        helpers.set_dummy_context()
        board = helpers.create_board()
        user = helpers.create_user()
        helpers.set_context(user)
        boards_manager = helpers.get_boards_manager()
        template = boards_manager.create_template_from_board(board, title, description, False)
        self.assertEqual(template.data.title, title)
        self.assertEqual(template.data.description, description)
        self.assertTrue(template.data.is_template)
        self.assertEqual(template.data.visibility, board_module.BOARD_PRIVATE)
        template = boards_manager.create_template_from_board(board, title, description, True)
        self.assertEqual(template.data.visibility, board_module.BOARD_PUBLIC)

    def test_switch_view(self):
        board = helpers.create_board()
        self.assertEqual(board.model, 'columns')
        board.switch_view()
        self.assertEqual(board.model, 'calendar')
        board.switch_view()
        self.assertEqual(board.model, 'columns')

    def test_has_member_1(self):
        """Test has member 1"""
        helpers.set_dummy_context()
        board = helpers.create_board()
        user = helpers.create_user('bis')
        helpers.set_context(user)
        board.add_member(user)
        self.assertTrue(board.has_member(user))

    # def test_has_member_2(self):
    #     """Test has member 2"""
    #     helpers.set_dummy_context()
    #     board = helpers.create_board()
    #     user = helpers.create_user('bis')
    #     helpers.set_context(user)
    #     user_2 = helpers.create_user(suffixe='2')
    #     board.add_member(user_2, role='manager')
    #     self.assertFalse(board.has_member(user))

    def test_has_manager_1(self):
        """Test has manager 1"""
        helpers.set_dummy_context()
        board = helpers.create_board()
        user = helpers.create_user('bis')
        helpers.set_context(user)
        self.assertFalse(board.has_manager(user))
        board.add_member(user, role='manager')
        self.assertTrue(board.has_manager(user))

    def test_has_manager_2(self):
        """Test has manager 2"""
        helpers.set_dummy_context()
        board = helpers.create_board()
        user = helpers.create_user('bis')
        helpers.set_context(user)
        user_2 = helpers.create_user(suffixe='2')
        self.assertFalse(board.has_manager(user))
        board.add_member(user_2, role='manager')
        database.session.flush()
        self.assertTrue(board.has_manager(user_2))
        self.assertFalse(board.has_manager(user))

    def test_add_member_1(self):
        """Test add member"""
        helpers.set_dummy_context()
        board = helpers.create_board()
        user = helpers.create_user('bis')
        helpers.set_context(user)
        self.assertFalse(board.has_member(user))
        board.add_member(user)
        self.assertTrue(board.has_member(user))

    def test_change_role(self):
        '''Test change role'''
        helpers.set_dummy_context()
        board = helpers.create_board()
        user = helpers.create_user('test')
        board.add_member(user)
        board.update_members()

        def find_board_member():
            for member in board.all_members:
                print member().user(), user
                if member().user() == user:
                    return member()

        member = find_board_member()
        self.assertEqual(len(board.members), 1)
        self.assertEqual(len(board.managers), 1)

        member.dispatch('toggle_role', '')
        member = find_board_member()
        board.update_members()
        self.assertEqual(len(board.members), 0)
        self.assertEqual(len(board.managers), 2)

        member.dispatch('toggle_role', '')
        board.update_members()
        self.assertEqual(len(board.members), 1)
        self.assertEqual(len(board.managers), 1)

    def test_get_boards(self):
        '''Test get boards methods'''

        helpers.set_dummy_context()
        board = helpers.create_board()
        user = helpers.create_user()
        user2 = helpers.create_user('bis')
        board.add_member(user2, 'member')
        boards_manager = helpers.get_boards_manager()
        self.assertTrue(board.has_manager(user))
        self.assertFalse(board.has_manager(user2))

        def in_comp(obj, comp_list):
            return any(obj == comp() for comp in comp_list)

        helpers.set_context(user)
        boards_manager.load_user_boards()
        self.assert_(not in_comp(board, boards_manager.last_modified_boards))
        self.assert_(not in_comp(board, boards_manager.guest_boards))
        self.assert_(in_comp(board, boards_manager.my_boards))
        self.assert_(not in_comp(board, boards_manager.archived_boards))

        helpers.set_context(user2)
        boards_manager.load_user_boards()
        self.assert_(not in_comp(board, boards_manager.last_modified_boards))
        self.assert_(in_comp(board, boards_manager.guest_boards))
        self.assert_(not in_comp(board, boards_manager.my_boards))
        self.assert_(not in_comp(board, boards_manager.archived_boards))

        column = board.create_column(1, u'test')
        column.create_card(u'test')
        boards_manager.load_user_boards()
        self.assert_(in_comp(board, boards_manager.last_modified_boards))

        board.archive()
        boards_manager.load_user_boards()
        self.assert_(in_comp(board, boards_manager.archived_boards))

    def test_get_by(self):
        '''Test get_by_uri and get_by_id methods'''
        helpers.set_dummy_context()
        orig_board = helpers.create_board()
        board = self.boards_manager.get_by_id(orig_board.id)
        self.assertEqual(orig_board.data.id, board.data.id)
        self.assertEqual(orig_board.data.title, board.data.title)
        board = self.boards_manager.get_by_uri(orig_board.data.uri)
        self.assertEqual(orig_board.data.id, board.data.id)
        self.assertEqual(orig_board.data.title, board.data.title)



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--
from nagare.i18n import _, _N, format_datetime
from nagare import ajax, presentation, security, var

from .comp import Comments, Comment, Commentlabel


@presentation.render_for(Comment)
@presentation.render_for(Comment, model='flow')
def render_comment(self, h, comp, model, *args):
    with h.div(class_='comment'):
        with h.div(class_='left'):
            h << self.author.render(h, model='avatar')
        with h.div(class_='right'):
            h << self.author.render(h, model='fullname')
            h << ' ' << _('wrote') << ' '
            h << h.span(
                _(u'on'), ' ',
                format_datetime(self.creation_date),
                class_="date")
            if security.has_permissions('delete_comment', self):
                onclick = (
                    u"if (confirm(%(message)s)){%(action)s;}return false" %
                    {
                        'action': h.a.action(
                            comp.answer, comp
                        ).get('onclick'),
                        'message': ajax.py2js(
                            _(u'Your comment will be deleted. Are you sure?')
                        ).decode('UTF-8')
                    }
                )
                h << h.a(h.i(class_='icon-cross'), title=_('Delete'),
                         class_="comment-delete", onclick=onclick, href='')
            h << self.comment_label.render(h.AsyncRenderer())
    return h.root


@presentation.render_for(Commentlabel)
def render_for_comment_label(self, h, comp, *args):
    """Render the text  of the associated comment

    """
    id_ = h.generate_id()
    with h.div(id=id_, class_='text'):
        a = h.div([h.p(part) if part else h.br for part in self.text.split('\n')])
        if security.has_permissions('edit_comment', self):
            with a:
                h << {'onclick': h.a.action(comp.answer).get('onclick')}
        h << a
    h << h.script("YAHOO.kansha.app.urlify($('#' + %s))" % ajax.py2js(id_))
    return h.root


@presentation.render_for(Commentlabel, model='edit')
def render_comment_label_form(self, h, comp, *args):
    """edit a comment """
    text = var.Var(self.text)
    with h.form:
        txt_id, buttons_id = h.generate_id(), h.generate_id()
        sub_id = h.generate_id()
        h << h.textarea(text(), class_='expanded', id_=txt_id).action(text)
        with h.div(class_='buttons', id=buttons_id):
            h << h.input(value=_("Save"), id=sub_id, type='submit',
                         class_="btn btn-primary").action(lambda: comp.answer(self.change_text(text())))
            h << ' '
            h << h.input(value=_("Cancel"), type='submit',
                         class_="btn").action(lambda: comp.answer(''))

    return h.root


@presentation.render_for(Comments, model='form')
def render_comments_form(self, h, comp, *args):
    """Add a comment to the current card"""
    if security.has_permissions('comment', self):
        text = var.Var()
        with h.form:
            txt_id, buttons_id = h.generate_id(), h.generate_id()
            sub_id = h.generate_id()
            kw = {
                "id": txt_id,
                "placeholder": _("Add comment."),
                "onfocus": "YAHOO.kansha.app.show('%s', true);YAHOO.util.Dom.addClass(this, 'expanded'); " % buttons_id,
            }
            h << h.textarea(**kw).action(text)
            h.head.javascript(
                h.generate_id(),
                'YAHOO.kansha.app.addCtrlEnterHandler(%s, %s)' % (
                    ajax.py2js(txt_id), ajax.py2js(sub_id)
                )
            )
            with h.div(id=buttons_id, class_="buttons hidden"):
                h << h.input(value=_("Save"), id=sub_id, type='submit',
                             class_="btn btn-primary").action(lambda: comp.answer(text()))
                h << ' '
                h << h.input(value=_("Cancel"), type='submit',
                             class_="btn").action(lambda: comp.answer(''))

    return h.root


@presentation.render_for(Comments, model='badge')
def render_comments_badge(self, h, *args):
    """Comment badge for the card"""
    num_comments = self.num_comments
    if num_comments:
        with h.span(class_='badge'):
            h << h.span(h.i(class_='icon-bubble2'), ' ', num_comments, class_='label')
    return h.root


@presentation.render_for(Comments)
def render(self, h, comp, *args):
    """Render card comments"""
    self.load_children()
    with h.div(class_='card-extension-comments'):
        h << h.div(comp.render(h, "badge"), class_='nbItems')
        h << comp.on_answer(self.add).render(h, 'form') << h.hr
        h << self.comments

    return h.root



# -*- coding:utf-8 -*-
# --
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

import uuid
import urllib

from elixir import using_options
from elixir import ManyToOne, OneToMany, OneToOne
from elixir import Field, Unicode, Integer, Boolean, UnicodeText

from kansha.models import Entity
from nagare.database import session
from kansha.user.models import DataUser
from kansha.column.models import DataColumn
# provisional until we have board extensions
from kansha.card_addons.label import DataLabel
# provisional until we have board extensions
from kansha.card_addons.weight import DataBoardWeightConfig
# provisional until we have board extensions
from kansha.card_addons.members.models import DataMembership

# Board visibility
BOARD_PRIVATE = 0
BOARD_PUBLIC = 1
BOARD_SHARED = 2


class DataBoard(Entity):
    """Board mapper

     - ``title`` -- board title
     - ``is_template`` -- is this a real board or a template?
     - ``columns`` -- list of board columns
     - ``labels`` -- list of labels for cards
     - ``comments_allowed`` -- who can comment ? (0 nobody, 1 board members only , 2 all application users)
     - ``votes_allowed`` -- who can vote ? (0 nobody, 1 board members only , 2 all application users)
     - ``description`` -- board description
     - ``visibility`` -- board visibility [0 Private, 1 Public (anyone with the URL can view),
                                           2 Shared (anyone can view it from her home page)]
     - ``uri`` -- board URI (Universally Unique IDentifier)
     - ``last_users`` -- list of last users
     - ``pending`` -- invitations pending for new members (use token)
     - ``archive`` -- display archive column ? (0 false, 1 true)
     - ``archived`` -- is board archived ?
    """
    using_options(tablename='board')
    title = Field(Unicode(255))
    is_template = Field(Boolean, default=False)
    columns = OneToMany('DataColumn', order_by="index",
                        cascade='delete', lazy='subquery')
    # provisional
    labels = OneToMany('DataLabel', order_by='index')
    comments_allowed = Field(Integer, default=1)
    votes_allowed = Field(Integer, default=1)
    description = Field(UnicodeText, default=u'')
    visibility = Field(Integer, default=0)
    version = Field(Integer, default=0, server_default='0')
    # provisional
    board_members = OneToMany('DataMembership', lazy='subquery', order_by=('manager'))
    uri = Field(Unicode(255), index=True, unique=True)
    last_users = ManyToOne('DataUser', order_by=('fullname', 'email'))
    pending = OneToMany('DataToken', order_by='username')
    history = OneToMany('DataHistory')

    background_image = Field(Unicode(255))
    background_position = Field(Unicode(255))
    title_color = Field(Unicode(255))
    show_archive = Field(Integer, default=0)
    archived = Field(Boolean, default=False)

    # provisional
    weight_config = OneToOne('DataBoardWeightConfig')

    def __init__(self, *args, **kwargs):
        """Initialization.

        Create board and uri of the board
        """
        super(DataBoard, self).__init__(*args, **kwargs)
        self.uri = unicode(uuid.uuid4())

    @property
    def template_title(self):
        manager = self.get_first_manager()
        if not manager or self.visibility == 0:
            return self.title
        return u'{0} ({1})'.format(self.title, manager.fullname)

    def get_first_manager(self):
        if not self.board_members:
            return None
        potential_manager = self.board_members[-1]
        return potential_manager.user if potential_manager.manager else None

    def copy(self):
        new_data = DataBoard(title=self.title,
                             description=self.description,
                             background_position=self.background_position,
                             title_color=self.title_color,
                             comments_allowed=self.comments_allowed,
                             votes_allowed=self.votes_allowed)
        # TODO: move to board extension
        new_data.weight_config = DataBoardWeightConfig(
            weighting_cards=self.weighting_cards,
            weights=self.weights)
        session.add(new_data)
        session.flush()
        # TODO: move to board extension
        for label in self.labels:
            new_data.labels.append(label.copy())
        session.flush()
        return new_data

    def get_label_by_title(self, title):
        return (l for l in self.labels if l.title == title).next()

    def delete_history(self):
        for event in self.history:
            session.delete(event)
        session.flush()

    def increase_version(self):
        self.version += 1
        if self.version > 2147483600:
            self.version = 1

    @property
    def url(self):
        return "%s/%s" % (urllib.quote_plus(self.title.encode('ascii', 'ignore').replace('/', '_')), self.uri)

    @classmethod
    def get_by_id(cls, id):
        return cls.get(id)

    @classmethod
    def get_by_uri(cls, uri):
        return cls.get_by(uri=uri)

    def set_background_image(self, image):
        self.background_image = image or u''

    @classmethod
    def get_all_boards(cls, user):
        """Return all boards the user is member of."""
        query = session.query(cls).join(DataMembership)
        query = query.filter(cls.is_template == False, DataMembership.user == user)
        return query.order_by(cls.title)

    @classmethod
    def get_shared_boards(cls):
        query = session.query(cls).filter(cls.visibility == BOARD_SHARED)
        return query.order_by(cls.title)

    @classmethod
    def get_templates_for(cls, user, public_value):
        q = cls.query
        q = q.filter(cls.archived == False)
        q = q.filter(cls.is_template == True)
        q = q.order_by(cls.title)

        q1 = q.filter(cls.visibility == public_value)

        q2 = q.join(DataMembership)
        q2 = q2.filter(DataMembership.user == user)
        q2 = q2.filter(cls.visibility != public_value)

        return q1, q2

    def create_column(self, index, title, nb_cards=None, archive=False):
        return DataColumn.create_column(self, index, title, nb_cards, archive)

    def delete_column(self, column):
        if column in self.columns:
            self.columns.remove(column)

    def create_label(self, title, color):
        label = DataLabel(title=title, color=color)
        self.labels.append(label)
        session.flush()
        return label

    ############# Membership management; those functions belong to a board extension

    def delete_members(self):
        DataMembership.delete_members(self)

    def has_member(self, user):
        """Return True if user is member of the board

        In:
         - ``user`` -- user to test (DataUser instance)
        Return:
         - True if user is member of the board
        """
        return DataMembership.has_member(self, user)

    def has_manager(self, user):
        """Return True if user is manager of the board

        In:
         - ``user`` -- user to test (DataUser instance)
        Return:
         - True if user is manager of the board
        """
        return DataMembership.has_member(self, user, manager=True)

    def remove_member(self, user):
        DataMembership.remove_member(board=self, user=user)

    def change_role(self, user, new_role):
        DataMembership.change_role(self, user, new_role == 'manager')

    def add_member(self, user, role='member'):
        """ Add new member to the board

        In:
         - ``new_member`` -- user to add (DataUser instance)
         - ``role`` -- role's member (manager or member)
        """
        DataMembership.add_member(self, user, role == 'manager')

    ############# Weight configuration, those functions belong to an extension

    @property
    def weights(self):
        return self.weight_config.weights

    @weights.setter
    def weights(self, value):
        self.weight_config.weights = value

    @property
    def weighting_cards(self):
        return self.weight_config.weighting_cards

    @weighting_cards.setter
    def weighting_cards(self, value):
        self.weight_config.weighting_cards = value

    def reset_card_weights(self):
        self.weight_config.reset_card_weights()

    def total_weight(self):
        return self.weight_config.total_weight()

# Populate
DEFAULT_LABELS = (
    (u'Green', u'#22C328'),
    (u'Red', u'#CC3333'),
    (u'Blue', u'#3366CC'),
    (u'Yellow', u'#D7D742'),
    (u'Orange', u'#DD9A3C'),
    (u'Purple', u'#8C28BD')
)


def create_template_empty():
    board = DataBoard(title=u'Empty board', is_template=True, visibility=1)
    board.weight_config = DataBoardWeightConfig()
    for title, color in DEFAULT_LABELS:
        board.create_label(title=title, color=color)
    session.flush()
    return board


def create_template_todo():
    board = DataBoard(title=u'Basic Kanban', is_template=True, visibility=1)
    board.weight_config = DataBoardWeightConfig()
    for index, title in enumerate((u'To Do', u'Doing', u'Done')):
        board.create_column(index, title)
    for title, color in DEFAULT_LABELS:
        board.create_label(title=title, color=color)
    session.flush()
    return board



#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from . import view
from .comp import (Board,
                   BOARD_PUBLIC, BOARD_PRIVATE,
                   COMMENTS_MEMBERS, COMMENTS_OFF,  COMMENTS_PUBLIC,
                   VOTES_MEMBERS, VOTES_OFF, VOTES_PUBLIC)



#-*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--
'''
Plugin based Seach Engine abstraction.
Available plugins are registered as entry points in distribution.
'''

# TODO: create plugins for whoosh, elasticsearch and solr

import pkg_resources


def SearchEngine(engine='dummy', **config):
    entry = pkg_resources.load_entry_point(
        'kansha', 'search.engines', engine)
    return entry(**config)



#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

import sys


APP = 'kansha'



global apps

app = apps[APP]

# call main
if len(sys.argv) < 3 or not sys.argv[1].isdigit():
    print 'Please provide the timespan (in hours, as integer) of the summary and the root URL of the app'
    sys.exit(0)
app.send_notifications(int(sys.argv[1]), sys.argv[2])



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from nagare import security
from peak.rules import when
from nagare.security import common

from kansha.card import Card
from kansha.cardextension import CardExtension
from kansha.board import VOTES_PUBLIC, VOTES_MEMBERS

from .models import DataVote


class Votes(CardExtension):
    '''Vote component'''

    LOAD_PRIORITY = 70

    @property
    def allowed(self):
        return self.configurator.votes_allowed

    def count_votes(self):
        '''Returns number of votes for a card'''
        return DataVote.count_votes(self.card.data)

    def toggle(self):
        '''Add a vote to the current card.

        Remove vote if user has already voted
        '''
        security.check_permissions('vote', self)
        user = security.get_user()
        if self.has_voted():
            DataVote.get_vote(self.card.data, user.data).delete()
        else:
            DataVote.new(card=self.card.data, user=user.data)

    def has_voted(self):
        '''Check if the current user already vote for this card'''
        user = security.get_user()
        return DataVote.has_voted(self.card.data, user.data)

    def delete(self):
        DataVote.purge(self.card.data)


# FIXME: redesign security from scratch
@when(common.Rules.has_permission, "user and perm == 'vote' and isinstance(subject, Votes)")
def has_permission_Card_vote(self, user, perm, votes):
    return ((security.has_permissions('edit', votes.card) and votes.allowed == VOTES_MEMBERS) or
            (votes.allowed == VOTES_PUBLIC and user))



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

import unittest

from kansha import validator


class ValidatorTest(unittest.TestCase):

    def test_boolean_validator(self):
        '''Test Boolean validator conversion type
        '''
        validate = validator.BoolValidator

        self.assertEqual(validate(True).value, True)
        self.assertEqual(validate(1).value, True)
        self.assertEqual(validate('true').value, True)
        self.assertEqual(validate('on').value, True)
        self.assertEqual(validate('yes').value, True)

        self.assertEqual(validate('test').value, False)
        self.assertEqual(validate(False).value, False)
        self.assertEqual(validate(0).value, False)
        self.assertEqual(validate('false').value, False)
        self.assertEqual(validate('off').value, False)
        self.assertEqual(validate('no').value, False)



"""Add templates

Revision ID: 2b0edcfa57b4
Revises: 24be36b8c67
Create Date: 2015-11-24 17:50:13.280722

"""

from alembic import op
import sqlalchemy as sa



# revision identifiers, used by Alembic.
revision = '2b0edcfa57b4'
down_revision = '24be36b8c67'

DEFAULT_LABELS = (
    (u'Green', u'#22C328'),
    (u'Red', u'#CC3333'),
    (u'Blue', u'#3366CC'),
    (u'Yellow', u'#D7D742'),
    (u'Orange', u'#DD9A3C'),
    (u'Purple', u'#8C28BD')
)


boards = sa.Table(
    'board',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('title', sa.Unicode(255)),
    sa.Column('is_template', sa.Boolean, default=False),
    sa.Column('comments_allowed', sa.Integer, default=1),
    sa.Column('votes_allowed', sa.Integer, default=1),
    sa.Column('description', sa.UnicodeText, default=u''),
    sa.Column('visibility', sa.Integer, default=0),
    sa.Column('version', sa.Integer, default=0, server_default='0'),
    sa.Column('uri', sa.Unicode(255), index=True, unique=True),
    sa.Column('archive', sa.Integer, default=0),
    sa.Column('archived', sa.Boolean, default=False),
    sa.Column('weighting_cards', sa.Integer, default=0),
    sa.Column('weights', sa.Unicode(255), default=u'')
)

columns = sa.Table(
    'column',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('title', sa.Unicode(200)),
    sa.Column('index', sa.Integer),
    sa.Column('nb_max_cards', sa.Integer),
    sa.Column('archive', sa.Boolean, default=False),
    sa.Column('board_id', sa.Integer)
)

labels = sa.Table(
    'label',
    sa.MetaData(),
    sa.Column('id', sa.Integer, primary_key=True),
    sa.Column('title', sa.Unicode(255)),
    sa.Column('color', sa.Unicode(255)),
    sa.Column('board_id', sa.Integer),
    sa.Column('index', sa.Integer)
)


def upgrade():
    bind = op.get_bind()

    # Add column
    op.add_column('board', sa.Column('is_template', sa.Boolean, default=lambda: False))

    bind.execute(boards.update().values(is_template=False))

    # Empty board
    insert = boards.insert({'title': u'Empty board',
                            'is_template': True,
                            'visibility': 1})
    board_id = bind.execute(insert).inserted_primary_key[0]
    for index, (title, color) in enumerate(DEFAULT_LABELS):
        insert = labels.insert({'title': title,
                                'color': color,
                                'index': index,
                                'board_id': board_id})
        bind.execute(insert)

    # Todo board
    insert = boards.insert({'title': u'Todo',
                            'is_template': True,
                            'visibility': 1})
    board_id = bind.execute(insert).inserted_primary_key[0]
    for index, title in enumerate((u'To Do', u'Doing', u'Done')):
        insert = columns.insert({'title': title,
                                 'index': index,
                                 'board_id': board_id})
        bind.execute(insert)

    for index, (title, color) in enumerate(DEFAULT_LABELS):
        insert = labels.insert({'title': title,
                                'color': color,
                                'index': index,
                                'board_id': board_id})
        bind.execute(insert)


def downgrade():
    op.drop_column('board', 'is_template')



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from nagare import ajax, component, editor, presentation, validator as nagare_validator
from nagare.i18n import _

from kansha import validator


class SaveTemplateEditor(object):
    def __init__(self, title, description, save_template_func):
        self.save_template_func = save_template_func
        self.title = editor.Property(title)
        self.title.validate(lambda v: nagare_validator.to_string(v).not_empty())
        self.description = editor.Property(description)
        self.shared = editor.Property(False).validate(validator.BoolValidator)

    def is_validated(self):
        return all(getattr(self, name).error is None for name in ('title', 'shared'))

    def cancel(self, comp):
        comp.answer(False)

    def commit(self, comp):
        if self.is_validated():
            comp.answer(True)

    def save(self):
        self.save_template_func(self.title.value, self.description.value, self.shared.value)


@presentation.render_for(SaveTemplateEditor)
def render_SaveTemplateEditor(self, h, comp, *args):
    h << h.h2(_(u'Save board as template'))
    with h.form(class_='description-form'):
        with h.div:
            id_ = h.generate_id()
            h << h.label(_(u'Title'), for_=id_)
            h << h.input(type='text', value=self.title(), autofocus=True).error(self.title.error).action(self.title)
        with h.div:
            id_ = h.generate_id()
            h << h.label(_(u'Description'), for_=id_)
            h << h.textarea(self.description()).action(self.description)

        with h.div:
            id_ = h.generate_id()
            h << h.label(_(u'Save for'), for_=id_)
            with h.select(id_=id_).error(self.shared.error).action(self.shared):
                h << h.option(_(u'Me only'), value='').selected(not self.shared())
                h << h.option(_(u'All users'), value='on').selected(self.shared())

        with h.div(class_='buttons'):
            h << h.button(_(u'Save'), class_='btn btn-primary', type='submit').action(self.commit, comp)
            h << ' '
            h << h.button(_('Cancel'), class_='btn').action(self.cancel, comp)
    return h.root


@presentation.render_for(SaveTemplateEditor, 'loading')
def render_SaveTemplateEditor_loading(self, h, comp, *args):
    id_ = h.generate_id()
    with h.div(class_='loading', id_=id_):
        h << h.img(src='img/ajax-loader.gif')
        h << _(u'Please wait while board is saved...')

    h << h.script(h.a.action(comp.answer).get('onclick').replace('return ', ''))
    return h.root


@presentation.render_for(SaveTemplateEditor, 'saved')
def render_SaveTemplateEditor_saved(self, h, comp, *args):
    with h.div(class_='success'):
        h << h.i(class_='icon-checkmark')
        h << _(u'Template saved!')
        with h.div(class_='buttons'):
            h << h.SyncRenderer().a(_(u'OK'), class_='btn btn-primary').action(comp.answer)
    return h.root


class SaveTemplateTask(component.Task):
    def __init__(self, title, description, save_template_func):
        self.title = title
        self.description = description
        self.save_template_func = save_template_func

    def go(self, comp):
        template_editor = SaveTemplateEditor(self.title, self.description, self.save_template_func)
        answer = comp.call(template_editor)
        if answer:
            comp.call(template_editor, 'loading')
            template_editor.save()
            comp.call(template_editor, 'saved')
        comp.answer()



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

import unittest
import os, shutil
import pkg_resources

from kansha.services.dummyassetsmanager import dummyassetsmanager
from kansha.services.simpleassetsmanager import simpleassetsmanager


class DummyAssetsManagerTest(unittest.TestCase):

    def setUp(self):
        self.dam = dummyassetsmanager.DummyAssetsManager()

    def test_load(self):
        """DummyAssetsManagerTest - Test load"""
        package = pkg_resources.Requirement.parse('kansha')
        fname = pkg_resources.resource_filename(
            package, 'kansha/services/dummyassetsmanager/tie.jpg')
        with open(fname, 'r') as f:
            data = f.read()

        res_data, res_metadata = self.dam.load('mock_id')
        self.assertEqual(res_data, data)
        self.assertEqual(res_metadata, {})

    def test_save(self):
        """DummyAssetsManagerTest - Test save"""
        package = pkg_resources.Requirement.parse('kansha')
        fname = pkg_resources.resource_filename(
            package, 'kansha/services/dummyassetsmanager/tie.jpg')
        with open(fname, 'r') as f:
            data = f.read()

        file_id = self.dam.save(data, {})
        self.assertEqual(file_id, 'mock_id')


class SimpleAssetsManagerTest(unittest.TestCase):

    def setUp(self):
        os.mkdir('/tmp/amtests')
        self.dam = simpleassetsmanager.SimpleAssetsManager(
            '', None, basedir='/tmp/amtests', baseurl='kansha', max_size=2048)

    def tearDown(self):
        shutil.rmtree('/tmp/amtests')

    def test_save_and_load(self):
        """SimpleAssetsManagerTest - Test save and load #1, save and load an image with metadata"""
        package = pkg_resources.Requirement.parse('kansha')
        test_file = pkg_resources.resource_filename(
            package, 'kansha/services/dummyassetsmanager/tie.jpg')
        with open(test_file, 'r') as f:
            data = f.read()

        file_id = self.dam.save(
            data, metadata={"id": "tie.jpg", "creation_date": "2013/01/02"})

        res_data, res_metadata = self.dam.load(file_id)
        self.assertEqual(res_data, data)
        self.assertEqual(
            res_metadata, {"id": "tie.jpg", "creation_date": "2013/01/02"})

    def test_save_and_load_2(self):
        """SimpleAssetsManagerTest - Test save and load #2, save and load an image without metadata"""
        package = pkg_resources.Requirement.parse('kansha')
        test_file = pkg_resources.resource_filename(
            package, 'kansha/services/dummyassetsmanager/tie.jpg')
        with open(test_file, 'r') as f:
            data = f.read()

        file_id = self.dam.save(data)

        res_data, res_metadata = self.dam.load(file_id)
        self.assertEqual(res_data, data)
        self.assertEqual(res_metadata, {})

    def test_save_and_load_3(self):
        """SimpleAssetsManagerTest - Test save and load #3, save and load an image with given id"""
        package = pkg_resources.Requirement.parse('kansha')
        test_file = pkg_resources.resource_filename(
            package, 'kansha/services/dummyassetsmanager/tie.jpg')
        with open(test_file, 'r') as f:
            data = f.read()

        file_id = self.dam.save(data, file_id="test.jpg")
        self.assertEqual(file_id, "test.jpg")
        res_data, _ = self.dam.load("test.jpg")
        self.assertEqual(res_data, data)

    def test_save_and_copy(self):
        package = pkg_resources.Requirement.parse('kansha')
        test_file = pkg_resources.resource_filename(
            package, 'kansha/services/dummyassetsmanager/tie.jpg')
        with open(test_file, 'r') as f:
            data = f.read()

        file_id = self.dam.save(data, file_id="test.jpg")
        new_id = self.dam.copy(file_id)
        res_data, _ = self.dam.load(new_id)
        self.assertEqual(res_data, data)



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

import calendar
from datetime import date, datetime
from dateutil import relativedelta
from nagare import ajax, presentation, i18n, var
from nagare import security
from nagare.i18n import _


YEARS_AROUND = 5


def to_date(v, format='short'):
    '''Format various inputs into a date object
    In:
        - v -- the value to transform
        - format -- if ``v`` is a string, provide the date format used in it
    '''
    if isinstance(v, datetime):
        return v.date()
    elif isinstance(v, (str, unicode)):
        try:
            return i18n.parse_date(v, format)
        except:
            raise ValueError(_('Invalid date format !'))
    return v


def in_(d, start, end):
    '''is ``d`` value between ``start`` and ``end``
    In:
        - d -- a date (or datetime) object'''
    return start <= d <= end


def before(d, start, strict=True):
    '''is ``d`` before ``start``
    In:
        - strict -- ``d`` must be before if True else before or equal
    '''
    if strict:
        return d < start
    return d <= start


class Calendar(object):

    def __init__(self, d=None, min_date=None, allow_none=False):
        self.date = d
        self.min_date = min_date
        if self.date is not None:
            val = self.date
        elif self.min_date is not None:
            val = to_date(self.min_date)
        else:
            val = date.today()
        self.current = val.replace(day=1)
        self.is_hidden = True
        self.allow_none = allow_none

    def toggle(self):
        self.is_hidden = not self.is_hidden

    def next_month(self):
        self.current += relativedelta.relativedelta(months=+1)

    def previous_month(self):
        self.current += relativedelta.relativedelta(months=-1)

    def choose_date(self, day, comp):
        self.toggle()
        self.date = self.current.replace(day=day)
        comp.answer(self.date)

    def remove_date(self, comp):
        self.toggle()
        self.date = None
        comp.answer(self.date)

    def is_authorized_date(self, d):
        if self.min_date is not None and before(d, self.min_date):
            return False
        return True

    def set_today(self):
        self.current = date.today().replace(day=1)

    def change_month(self, month):
        self.current = self.current.replace(month=int(month))

    def change_year(self, year):
        self.current = self.current.replace(year=int(year))

    def __call__(self, value=None):
        if value is not None:
            self.date = to_date(value)
        return self.date


@presentation.render_for(Calendar)
def render_async(self, h, comp, *args):
    with h.div(class_='calendar-input'):
        input_id = h.generate_id('input')
        calendar_id = h.generate_id('calendar')
        if self.is_hidden:
            style = 'display: none;'
        else:
            style = 'display: block;'
        with h.div(class_='calendar', id_=calendar_id, style=style):
            with h.div(class_='calendar-header'):
                with h.a(title=_(u'Previous')).action(self.previous_month):
                    h << h.i(class_='icon-arrow-left', title=_(u'Previous'))
                with h.span(class_='current'):
                    with h.select(onchange=ajax.Update(action=self.change_month)):
                        for n, month in i18n.get_month_names().iteritems():
                            month = month.capitalize()
                            h << h.option(month, value=n).selected(self.current.month)
                    h << u' '
                    with h.select(onchange=ajax.Update(action=self.change_year)):
                        for year in xrange(self.current.year - YEARS_AROUND, self.current.year + YEARS_AROUND):
                            h << h.option(year, value=year).selected(self.current.year)

                with h.a(title=_(u'Next')).action(self.next_month):
                    h << h.i(class_='icon-arrow-right', title=_(u'Next'))

            with h.div(class_='calendar-content'):
                if isinstance(self.date, date):
                    if self.current.year == self.date.year and self.current.month == self.date.month:
                        active = self.date.day
                    else:
                        active = -1
                else:
                    active = -1
                today = date.today()
                if today.month == self.current.month and today.year == self.current.year:
                    today = today.day
                else:
                    today = -1

                with h.table:
                    with h.thead:
                        with h.tr:
                            h << h.th(h.span(_('Wk'), title=_('Week number')), class_='week_number')

                            days = [day.capitalize() for day in i18n.get_day_names().itervalues()]
                            h << [h.th(h.span(d[:2], title=d)) for d in days]
                    with h.tbody:
                        for line in calendar.monthcalendar(self.current.year, self.current.month):
                            with h.tr:
                                week_number = date(self.current.year, self.current.month, max(1, line[0])).isocalendar()[1]
                                h << h.td(week_number, class_='week_number')

                                for day in line:
                                    if day == 0:
                                        h << h.td(class_='not-this-month')
                                    else:
                                        authorized = self.is_authorized_date(self.current.replace(day=day))
                                        cls = []
                                        if day == active:
                                            cls.append(u'active')
                                        if day == today:
                                            cls.append(u'today')
                                        if not authorized:
                                            cls.append(u'excluded')
                                        with h.td(class_=' '.join(cls)):
                                            if authorized:
                                                h << h.a(day).action(lambda day=day: self.choose_date(day, comp))
                                            else:
                                                h << h.span(day)
            with h.div(class_='calendar-today'):
                h << h.a(h.i(class_='icon-calendar'), _(u"Today"), class_='today-link btn').action(self.set_today)
                if self.allow_none:
                    h << h.a(h.i(class_='icon-cross'), _(u'None'), class_='erase btn').action(lambda: self.remove_date(comp))

            h << h.script(
                "YAHOO.util.Event.onDOMReady(function() {"
                "var region = YAHOO.util.Dom.getRegion(%(input_id)s);"
                "YAHOO.util.Dom.setXY(%(calendar_id)s, [region.left, region.bottom + 3]);"
                "});" % {
                    'input_id': ajax.py2js(input_id),
                    'calendar_id': ajax.py2js(calendar_id)
                }
            )

            h << h.div(class_='clear')
    return h.root



"""membership

Revision ID: 25739bc150b9
Revises: 2c0a3788227
Create Date: 2017-01-13 17:58:55.516698

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '25739bc150b9'
down_revision = '2c0a3788227'

def upgrade():
    bind = op.get_bind()

    members = {}

    # gather board memberships
    select = sa.text('SELECT board_id, user_username, user_source FROM user_managed_boards__board_managers')
    for board_id, username, source in bind.execute(select):
        key = (username, source, board_id)
        members[key] = dict(board_id=board_id, user_username=username, user_source=source, manager=True, notify=0)

    select = sa.text('SELECT board_id, user_username, user_source, notify FROM user_boards__board_members')
    for board_id, username, source, notify in bind.execute(select):
        key = (username, source, board_id)
        if key not in members:
            members[key] = dict(board_id=board_id, user_username=username, user_source=source, manager=False, notify=notify)
        else:
            members[key]['notify'] = notify
    print sorted(members.keys())

    # Create unified membership table
    op.create_table(
        'membership',
        sa.Column('id', sa.Integer, primary_key=True, autoincrement=True),
        sa.Column('board_id', sa.Integer, sa.ForeignKey('board.id', ondelete='CASCADE'), nullable=False),
        sa.Column('user_username', sa.Unicode, nullable=False),
        sa.Column('user_source', sa.Unicode, nullable=False),
        sa.Column('manager', sa.Boolean, default=False, nullable=False),
        sa.Column('notify', sa.Integer, default=1),
        sa.ForeignKeyConstraint(['user_username', 'user_source'], ['user.username', 'user.source'], ondelete='CASCADE'),
        sa.UniqueConstraint('board_id', 'user_username', 'user_source', name='membership_ix')
    )
    membership = sa.Table(
        'membership',
        sa.MetaData(),
        sa.Column('id', sa.Integer, primary_key=True, autoincrement=True),
        sa.Column('board_id', sa.Integer, sa.ForeignKey('board.id', ondelete='CASCADE'), nullable=False),
        sa.Column('user_username', sa.Unicode, nullable=False),
        sa.Column('user_source', sa.Unicode, nullable=False),
        sa.Column('manager', sa.Boolean, default=False, nullable=False),
        sa.Column('notify', sa.Integer, default=1),
        sa.ForeignKeyConstraint(['user_username', 'user_source'], ['user.username', 'user.source'], ondelete='CASCADE'),
        sa.UniqueConstraint('board_id', 'user_username', 'user_source', name='membership_ix')
    )
    # migrate the data
    op.bulk_insert(membership, members.values())
    # Keep track of ids
    for ms in bind.execute(membership.select()):
        members[(ms.user_username, ms.user_source, ms.board_id)]['membership'] = ms.id

    # Create card membership table
    op.create_table(
        'membership_cards__card',
        sa.Column('id', sa.Integer, primary_key=True, autoincrement=True),
        sa.Column('membership_id', sa.Integer, sa.ForeignKey('membership.id', ondelete='CASCADE'), nullable=False),
        sa.Column('card_id', sa.Integer, sa.ForeignKey('card.id', ondelete='CASCADE'), nullable=False),
        sa.UniqueConstraint('membership_id', 'card_id', name='card_membership_ix')
    )

    card_membership = sa.Table(
        'membership_cards__card',
        sa.MetaData(),
        sa.Column('id', sa.Integer, primary_key=True, autoincrement=True),
        sa.Column('membership_id', sa.Integer, sa.ForeignKey('membership.id', ondelete='CASCADE'), nullable=False),
        sa.Column('card_id', sa.Integer, sa.ForeignKey('card.id', ondelete='CASCADE'), nullable=False),
        sa.UniqueConstraint('membership_id', 'card_id', name='card_membership_ix')
    )
    # gather card membership information
    card_members = []
    select = sa.text("SELECT cm.card_id, cm.user_username, cm.user_source, col.board_id "
                     "FROM user_cards__card_members as cm join card on cm.card_id = card.id "
                     'join "column" as col on card.column_id = col.id')
    for card_id, username, source, board_id in bind.execute(select):
        member = members.get((username, source, board_id))
        if member:
            card_members.append(dict(card_id=card_id, membership_id=member['membership']))
        else:
            print 'illegal member', username, 'on card', card_id
    # migrate the data
    op.bulk_insert(card_membership, card_members)

    op.drop_table('user_cards__card_members')
    op.drop_table('user_boards__board_members')
    op.drop_table('user_managed_boards__board_managers')


def downgrade():
    print 'There is no way back...'


#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from nagare.i18n import _
from nagare import presentation, security, var, ajax

from .comp import Checklist, ChecklistItem, Checklists, NewChecklistItem


@presentation.render_for(NewChecklistItem)
def render_NewCheclistItem(self, h, comp, model):
    return comp.render(h, 'button' if self.show_button else 'edit')


@presentation.render_for(NewChecklistItem, 'button')
def render_NewChecklistItem_button(self, h, comp, model):
    return h.a(h.i(class_='icon-plus'), u' ', _(u'Add item'), class_='add-item', title=_(u'Add item')).action(self.set_show_button, False)


@presentation.render_for(NewChecklistItem, 'edit')
def render_NewChecklistItem_edit(self, h, comp, model):
    """Render the title of the associated object"""
    text = var.Var(u'')
    with h.form(class_='new-item-form'):
        id_ = h.generate_id()
        h << h.input(type='text', value=text, id_=id_, placeholder=_(u'Add item')).action(text)
        h << h.button(_(u'Add'),
                      class_='btn btn-primary').action(lambda: comp.answer(text()))

    if self.focus:
        h << h.script("YAHOO.util.Dom.get(%s).focus()" % ajax.py2js(id_))
        self.focus = False
    return h.root


@presentation.render_for(Checklists, 'action')
def render_Checklists_button(self, h, comp, model):
    if security.has_permissions('checklist', self.card):
        action = ajax.Update(render=lambda r: comp.render(r, model=None),
                             component_to_update='clists',
                             action=self.add_checklist)
        with h.a(class_='btn').action(action):
            h << h.i(class_='icon-list')
            h << _('Checklist')
    return h.root


@presentation.render_for(Checklists)
def render_Checklists(self, h, comp, model):
    h.head.javascript_url('checklists/js/checklists.js')
    self.load_children()
    can_edit = security.has_permissions('checklist', self.card)
    with h.div(id_='clists'):
        if can_edit:

            # On drag and drop
            action = h.a.action(ajax.Update(action=self.reorder, with_request=True)).get('onclick').replace('return', '')
            action = action.replace('")', '&data="+ YAHOO.lang.JSON.stringify(data))')
            h.head.javascript(h.generate_id(), '''function reorder_checklists(data) {
                %s;
            }''' % action)

            # On items drag and drop
            action = h.a.action(ajax.Update(action=self.reorder_items, with_request=True)).get('onclick').replace('return', '')
            action = action.replace('")', '&data="+ YAHOO.lang.JSON.stringify(data))')
            h.head.javascript(h.generate_id(), '''function reorder_checklists_items(data) {
                    %s;
                }''' % action)

        id_ = h.generate_id()
        with h.div(class_='checklists', id=id_):
            not_alone = len(self.checklists) > 1
            for index, clist in enumerate(self.checklists):
                if can_edit:
                    h << clist.on_answer(
                        lambda v, index=index: self.delete_checklist(index)
                    ).render(h, 'not-alone' if not_alone else None)
                else:
                    h << clist.render(h, 'read-only')
    return h.root


@presentation.render_for(Checklists, 'badge')
def render_Checklists_badge(self, h, comp, model):
    total_items = self.total_items
    if total_items:
        with h.span(class_='badge'):
            h << h.span(h.i(class_='icon-list'), ' ', self.total_items_done, ' / ', total_items, class_='label')
    return h.root


@presentation.render_for(Checklist)
@presentation.render_for(Checklist, 'not-alone')
@presentation.render_for(Checklist, 'read-only')
def render_Checklist(self, h, comp, model):
    with h.div(id='checklist_%s' % self.id, class_='checklist'):
        with h.div(class_='title'):
            h << h.i(class_='icon-list ' + (model or ''))
            if model == 'read-only':
                h << self.data.title
            else:
                h << self.title.render(h.AsyncRenderer())
                onclick = (
                    u"if (confirm(%(message)s)){%(action)s;}return false" %
                    {
                        'action': h.a.action(
                            ajax.Update(render='', action=comp.answer)
                        ).get('onclick'),
                        'message': ajax.py2js(
                            _(u'The checklist will be deleted. Are you sure?')
                        ).decode('UTF-8')
                    }
                )
                h << h.a(h.i(class_='icon-cross'), class_='delete', onclick=onclick)

        with h.div(class_='content'):
            if self.items:
                h << comp.render(h, 'progress')
            with h.ul:
                not_alone = len(self.items) > 1
                for index, item in enumerate(self.items):
                    if model == 'read-only':
                        h << h.li(item.render(h, 'read-only'))
                    else:
                        h << h.li(
                            item.on_answer(
                                lambda v, index=index: self.delete_index(index)
                            ).render(h, model='not-alone' if not_alone else None),
                            id='checklist_item_%s' % item().id,
                            class_='checklist-item')
            if model != 'read-only':
                h << self.new_item
    return h.root


@presentation.render_for(Checklist, 'progress')
def render_Checklist_progress(self, h, comp, model):
    progress = self.progress
    with h.div(class_='progress progress-success'):
        h << h.div(class_='bar', style='width:%s%%' % progress)
        h << h.span(progress, u'%', class_='percent')
    return h.root


@presentation.render_for(ChecklistItem, model='read-only')
def render_ChecklistItem(self, h, comp, model):
    h << h.span(h.i(class_='icon-checkbox-' + ('checked' if self.done else 'unchecked')))
    h << h.span(self.data.title, class_='done' if self.done else '')
    return h.root


@presentation.render_for(ChecklistItem)
@presentation.render_for(ChecklistItem, 'not-alone')
def render_ChecklistItem(self, h, comp, model):
    with h.a().action(self.set_done):
        help_msg = _(u'Click to toggle')
        h << {'title': help_msg}
        with h.i:
            check_class = 'icon-checkbox-' + ('checked' if self.done else 'unchecked')
            h << {'class': check_class}
    h << h.span(self.title.render(h.AsyncRenderer()), class_='title ' + ('done' if self.done else ''))
    onclick = (
        u"if (confirm(%(message)s)){%(action)s;}return false" %
        {
            'action': h.a.action(
                ajax.Update(render='', action=lambda: comp.answer('delete'))
            ).get('onclick'),
            'message': ajax.py2js(
                _(u'The checklist item will be deleted. Are you sure?')
            ).decode('UTF-8')
        }
    )
    h << h.a(h.i(class_='icon-cross'), class_='delete-item', onclick=onclick)
    if model == 'not-alone':
        h << h.i(class_='icon-menu2 not-alone', title=_(u'Drag to re-order'))
    return h.root



# -*- coding:utf-8 -*-
# --
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
# --

import pkg_resources
import random
import os
import string
from cStringIO import StringIO

# PIL
from PIL import Image
from PIL import ImageFont
from PIL import ImageDraw

from nagare import presentation

APP_NAME = 'kansha'
_PACKAGE = pkg_resources.Requirement.parse(APP_NAME)


def get_resource_filename(path):
    return pkg_resources.resource_filename(_PACKAGE, path)  # @UndefinedVariable


def random_text(length=5):
    chars = string.ascii_uppercase + string.digits
    return ''.join(random.choice(chars) for dummy in range(length))


class Captcha(object):
    """Render a captcha image for the given text"""

    def __init__(self, text='', background_name='captcha-bg.png', font_name='ashcanbb_bold.ttf', font_size=30):
        self.text = random_text()
        self.background_path = get_resource_filename(
            os.path.join('data', 'captcha', background_name))
        self.font_path = get_resource_filename(
            os.path.join('data', 'captcha', font_name))
        self.font_size = font_size

    def _generate_image(self):
        """Generate the captcha image"""
        font = ImageFont.truetype(self.font_path, self.font_size)
        image = Image.open(self.background_path)
        image = image.convert("RGBA")
        for i, l in enumerate(self.text):
            txt = Image.new('RGBA', (80, 50), (0, 0, 0, 0))
            d = ImageDraw.Draw(txt)
            d.text((0, 0), l, font=font, fill=(9, 92, 110, 255))
            txt = txt.rotate(random.randint(-20, 20))
            image.paste(txt, (30 + 30 * i, 5), txt)
        return image

    def get_image_data(self):
        """Get the image data"""
        image = self._generate_image()
        out = StringIO()
        image.save(out, 'PNG')
        return out.getvalue()


@presentation.render_for(Captcha)
def render_captcha(self, h, comp, *args):
    h << h.img.action(self.get_image_data)
    return h.root



# -*- coding:utf-8 -*-
#--
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

import re
import string
import ntpath

from lxml import html
from nagare import i18n
from nagare import validator
from nagare.i18n import _, _L
from lxml.html.clean import Cleaner


# mail regex extracted from http://www.regular-expressions.info/email.html
MAIL_RE = r"(?i)^[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?$"


def clean_text(text):
    text = text.replace(u'<br>', u'<br>\n')
    text = text.replace(u'</p>', u'</p>\n')
    text = text.replace(u'</li>', u'</li>\n')

    return unicode(html.fromstring(text).text_content())


def clean_html(text):
    cleaner = Cleaner(style=False)
    return cleaner.clean_html(text)


class BoolValidator(validator.Validator):
    """Conversion and validation of integers
    """

    def __init__(self, value, strip=True, *args, **kw):
        """Initialisation

        Check that the value is a bool

        In:
          - ``v`` -- value to validate
        """
        try:
            if isinstance(value, basestring):
                value = value.strip()

                if value.lower() in ('yes', 'on', 'true', '1'):
                    self.value = True
                elif value.lower() in ('no', 'off', 'false', '0'):
                    self.value = False
                else:
                    self.value = False
            else:
                self.value = bool(value)
        except (ValueError, TypeError):
            raise ValueError(i18n._(u'Must be a boolean'))

    to_bool = validator.Validator.__call__


def validate_identifier(value, start_with_uppercase_letter=False, max_len=256):
    """Validate an identifier: only letters, digits or underscore characters are accepted"""
    if not value:
        raise ValueError(_("Should not be empty"))

    safe_chars = set("_%s%s" % (string.ascii_letters, string.digits))
    ok = all(c in safe_chars for c in value)
    if not ok:
        raise ValueError(_('Should only contain ascii letters, digits or underscore characters'))

    if start_with_uppercase_letter:
        if value[0] not in string.ascii_uppercase:
            raise ValueError(_("Should start with an uppercase ascii letter"))
    else:
        if value[0] not in string.ascii_letters:
            raise ValueError(_("Should start with an ascii letter"))

    if len(value) > max_len:
        raise ValueError(
            _("Identifier too long (max. %d characters)") % max_len)

    return value


def validate_file(data, max_size=None, msg=_('Size must be less than %d KB')):
    """Validate a 'file' input data against the max size in KB"""
    # check against the first roundtrip with the client
    if data is None or isinstance(data, basestring):
        return None

    if data.done == -1:
        raise ValueError(_('Transfer was interrupted'))

    # gets the file size (it's a StringIO)
    data.file.seek(0, 2)  # 0 from EOF
    filesize = data.file.tell()
    data.file.seek(0)

    if max_size is not None and filesize > max_size * 1024:
        raise ValueError(msg % max_size)

    filedata = data.file.read()
    data.file.seek(0)

    # some browsers (i.e. Internet Explorer) send the full path of the file
    # instead of the filename
    # so, we remove the path from the filename
    filename = ntpath.basename(unicode(data.filename))

    return {'filename': filename,
            'data': filedata,
            'content_type': unicode(data.type)}


def validate_non_empty_string(value, msg=_L("Required field")):
    """Check that the value is a non-empty string"""
    # strip whitespace characters
    return validator.StringValidator(value, strip=True).not_empty(_(msg)).to_string()


def validate_email(value, required_msg=_L("Required field"), invalid_msg=_L("Invalid email address")):
    """Check that the value is a valid email address"""
    if not value:
        raise ValueError(_(required_msg))

    # convert to lowercase
    value = value.lower()

    if len(value) > 7:
        if re.match(MAIL_RE, value):
            return value

    raise ValueError(_(invalid_msg))


def validate_password(value):
    value = validate_non_empty_string(value)

    # check password complexity
    min_len = 6
    if len(value) < min_len:
        raise ValueError(_("Password too short: should have at least %d characters")
                         % min_len)

    return value



# -*- coding:utf-8 -*-
# --
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
# --

import dateutil.parser

from nagare import component, security

from kansha import title
from kansha import events
from kansha.services.search import schema
from kansha.services.actionlog.messages import render_event

from .models import DataCard


class NewCard(object):

    """New card component
    """

    def __init__(self, column):
        self.column = column
        self.needs_refresh = False

    def toggle_refresh(self):
        self.needs_refresh = not self.needs_refresh


class Card(events.EventHandlerMixIn):

    """Card component
    """
    schema = schema.Schema('Card',
                           schema.Text(u'title', stored=True),
                           schema.Int('board_id'),
                           schema.Boolean('archived'))

    def __init__(self, id_, card_extensions, action_log, card_filter,
                 services_service, data=None):
        """Initialization

        In:
            - ``id_`` -- the id of the card in the database
            - ``column`` -- father
        """
        self.db_id = id_
        self.id = 'card_' + str(self.db_id)
        self.card_extensions = card_extensions
        self.action_log = action_log.for_card(self)
        self.card_filter = card_filter
        self._services = services_service
        self._data = data
        self.extensions = ()
        self.refresh()

    def add_to_index(self, search_engine, board_id, update=False):
        data = {'docid': self.id,
                'title': self.get_title(),
                'board_id': board_id,
                'archived': self.archived}
        document = self.schema(**data)
        for name, extension in self.extensions:
            extension().update_document(document)
        if update:
            search_engine.update_document(document)
        else:
            search_engine.add_document(document)
        # Don't forget to commit after

    @classmethod
    def update_schema(cls, card_extensions):
        for name, extension in card_extensions.iteritems():
            field = extension.get_schema_def()
            if field is not None:
                cls.schema.add_field(field)

    def update(self, other):
        self.data.update(other.data)
        # extensions must align
        extensions = zip(self.extensions, other.extensions)
        for (name, my_extension), (name2, other_extension) in extensions:
            assert(name == name2)  # should never raise
            my_extension().update(other_extension())

    def refresh(self):
        """Refresh the sub components
        """
        self.title = component.Component(
            title.EditableTitle(self.get_title)).on_answer(self.set_title)
        self.extensions = [
            (name, component.Component(extension))
            for name, extension in self.card_extensions.instantiate_items(self, self.action_log, self._services)
        ]

    @property
    def data(self):
        """Return the card object from the database
        PRIVATE
        """
        if self._data is None:
            self._data = DataCard.get(self.db_id)
        return self._data

    def __getstate__(self):
        self._data = None
        return self.__dict__

    @property
    def archived(self):
        return self.data.archived

    @property
    def index(self):
        return self.data.index

    def can_edit(self, user):
        """
        Has the user the permission to edit this card?
        """
        # if self.extensions is empty, ``all`` returns True, which is what we expect.
        return all(extension().has_permission_on_card(user, 'edit') for __, extension in self.extensions)

    def set_title(self, title):
        """Set title

        In:
            - ``title`` -- new title
        """
        values = {'from': self.data.title, 'to': title}
        self.action_log.add_history(security.get_user(), u'card_title', values)
        self.data.title = title

    def get_title(self):
        """Get title

        Return :
            - the card title
        """
        return self.data.title

    def delete(self):
        """Prepare for deletion"""
        for __, extension in self.extensions:
            extension().delete()
        self.action_log.delete_card()

    def card_dropped(self, request, response):
        """
        Dropped on new date (calendar view).
        """
        start = dateutil.parser.parse(request.GET['start']).date()
        for __, extension in self.extensions:
            extension().new_card_position(start)

    def emit_event(self, comp, kind, data=None):
        if kind == events.PopinClosed:
            kind = events.CardEditorClosed
        return super(Card, self).emit_event(comp, kind, data)



# -*- coding:utf-8 -*-
# --
# Copyright (c) 2012-2014 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
# --

#FIXME: this should be a board extension

from __future__ import absolute_import

import urlparse
import sqlalchemy as sa

from nagare import database
from nagare.i18n import _, _L
from nagare.namespaces import xhtml

from kansha.card_addons.members import Membership


# Levels
NOTIFY_OFF = 0
NOTIFY_MINE = 1
NOTIFY_ALL = 2


# Groups
GROUP_EVENTS = (
    ('affected', ('card_add_member', 'card_remove_member')),
    ('modified', ('card_move', 'card_title', 'card_add_file', 'card_add_comment', 'card_weight',
                  'card_add_list', 'card_delete_list', 'card_listitem_done', 'card_listitem_undone')),
    ('add_remove', ('card_create', 'card_delete', 'card_archive'))
)

GROUP_MESSAGES = {
    'affected': _L(u'Cards affectation:'),
    'modified': _L(u'Cards modification:'),
    'add_remove': _L(u'Cards addition / removal:')
}


def allow_notifications(user, board, level):
    member = Membership.search(board, user)
    if member:
        member.notify = level


def notifications_allowed(user, board):
    member = Membership.search(board, user)
    return member.notify if member else NOTIFY_OFF


def get_subscribers():
    return Membership.subscribers()


# FIXME: subscriber should be a business object, not a data
def filter_events(events, subscriber):
    if subscriber.notify == NOTIFY_ALL:
        return events
    elif subscriber.notify == NOTIFY_MINE:
        user_cards = set([card.id for card in subscriber.cards])
        return [event for event in events if event.card_id in user_cards]
    return []


# renders

# FIXME: use business objects, not raw data
def generate_email(app_title, board, user, hours, url, events):
    ret = []
    data = {'board': board.title, 'hours': hours, 'url': urlparse.urljoin(
        url, board.url), 'count': len(events), 'app': app_title}
    subject = _(
        u'[%(app)s] Board "%(board)s" - %(count)s new action(s)') % data

    h = xhtml.Renderer()
    root = h.div

    ret.append(
        _(u'The following actions have been made on the board "%(board)s" in the last %(hours)s hours:\n') % data)
    root.append(h.p(_(u'The following actions have been made on the board '),
                    h.a(data['board'], href=data['url']),
                    _(u' in the last %(hours)s hours:') % data))

    groups = {}
    for group, evts in GROUP_EVENTS:
        for event in evts:
            groups[event] = group

    events_by_group = {}
    for e in events:
        events_by_group.setdefault(groups[e.action], []).append(e)

    for group, events in GROUP_EVENTS:
        events = events_by_group.get(group)
        if events:
            with h.div:
                root.append(h.p(
                    _(GROUP_MESSAGES[group]), style='text-decoration: underline; font-weight: bold;'))
                with h.ul:
                    for event in events:
                        if event.card:
                            # IDs are interpreted as anchors since HTML4. So don't use the ID of
                            # the card as a URL fragment, because the browser
                            # jumps to it.
                            event = h.a(event.to_string(), href='%s#id_card_%s' % (
                                data['url'], event.card.id), style='text-decoration: none;')
                        else:
                            event = event.to_string()
                        root.append(h.li(event))

            ret.append(_(GROUP_MESSAGES[group]))
            ret.append('')
            for event in events:
                ret.append(u'- ' + event.to_string())
            ret.append(u'')

    ret.append(
        _(u'To unsubscribe from board "%(board)s" alerts, click on %(url)s and modify notifications parameters in the "settings" menu of the "Board" tab.') % data)
    root.append(h.p(_(u'To unsubscribe from board "%(board)s" alerts, ') % data,
                    h.a(_(u'click here'), href=data['url']),
                    _(' and modify notifications parameters in the "settings" menu of the "Board" tab.') % data, style='border: 1px solid black; padding: 5px; text-align: center;'))

    return subject, u'\n'.join(ret), root.write_htmlstring()
